#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'en_meps.dbxml';
my $output_dir = "$root_dir/mediawiki/meps_groups";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically en_meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: en_meps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/meps_groups
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

my %groups_order = (
  'PPE-DE'    => 1,
  'PSE'       => 2,
  'ALDE'      => 3,
  'UEN'       => 4,
  'Verts/ALE' => 5,
  'GUE/NGL'   => 6,
  'IND/DEM'   => 7,
  'NI'        => 8,
);

my %meps;

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query = <<"QUERY";
for \$group in distinct-values(collection("dbxml:/$theContainer.dbxml")/politician/infos/group/abbreviation)
return
  <result>
    <code>{\$group}</code>
    {
    for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[infos/group/abbreviation/string()=\$group]
    order by \$deputy/infos/constituency/country/code/string(), upper-case(\$deputy/infos/name/last/string()), upper-case(\$deputy/infos/name/first/string())
    return
      <deputy>
        <group>{ \$deputy/infos/group/name/string() }</group>
        <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "/en|", \$deputy/infos/name/full/string(), "]]") }</name>
        <country>{ \$deputy/infos/constituency/country/code/string() }</country>
        <role>{ \$deputy/infos/group/\@role }</role>
        <tel>{ string-join(for \$tel in \$deputy/contact//phone return \$tel/string(), "<br />") }</tel>
        <fax>{ string-join(for \$fax in \$deputy/contact//fax return \$fax/string(), "<br />") }</fax>
        <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "@", "(at)"), "]"), "<br />") }</email>
        <bxl>{ \$deputy/contact/address[label="Parlement européen"][city="Bruxelles"] }</bxl>
        <str>{ \$deputy/contact/address[label="Parlement européen"][city="Strasbourg Cedex"] }</str>
        {\$deputy/functions}
        {\$deputy/scores}
      </deputy>
    }
  </result>
QUERY
    #  Perform the query
    eval {
        my $results = $theMgr->query($query);
        my $value;
        while( $results->next($value) ) {
          my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
          my $code =  $xml->root->first_child_text('code');
          foreach my $mp ($xml->root->children('deputy')) {
            $meps{$code}->{NAME} = $mp->first_child_text('group');
            my %mp_data;

            # Skip meps not anti-three-stikes-approach
            my $score_bono;
            next unless $mp->first_child('scores');
            foreach my $score ($mp->first_child('scores')->children('score')) {
              next unless $score->first_child_text('wiki') eq 'Cultural_industries_resolution';
              $score_bono = $score->first_child_text('value');
            }
            next unless (defined $score_bono and $score_bono == 20);
            
            # Skip meps not member of ITRE or IMCO
            foreach my $com ($mp->first_child('functions')->children('function')) {
              my $abbrev = $com->first_child_text('abbreviation');
              next unless $abbrev eq 'ITRE' or $abbrev eq 'IMCO';

              my $committee = "[[MEPs_$abbrev|$abbrev]]";

              if ($com->first_child_text('role') eq 'Substitute') {
                $committee =~ s/\|([^\]]+)\]/|''$1'']/;
              }
              elsif ($com->first_child_text('role') ne 'Member') {
                my $com_role = $com->first_child_text('role');
                $committee =~ s/\|([^\]]+)\]/|$1 ($com_role)]/;
              }

              if ($mp_data{COM}) {
                $mp_data{COM} .= " / $committee";
              }
              else {
                $mp_data{COM} = $committee;
              }
            }
            next unless $mp_data{COM};

            $mp_data{NAME} = $mp->first_child_text('name');
            $mp_data{COUNTRY} = $mp->first_child_text('country');
            $mp_data{TEL} = $mp->first_child_text('tel');
            $mp_data{FAX} = $mp->first_child_text('fax');
            $mp_data{EMAIL} = $mp->first_child_text('email');
            my $bxl_addr = $mp->first_child('bxl')->first_child('address');
            $mp_data{BXL} = $bxl_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $bxl_addr->first_child_text('office');
            if ($mp_data{BXL} eq '- Moving') {
              $mp_data{BXL} = '[[MEPs_BXL_SDF|-]]';
            }
            else {
              $mp_data{BXL} =~ s/^(([A-Z][A-Z][A-Z]) (\d\d).+)$/[[MEPs_$2_$3|$1]]/o;
            }
            
            my $str_addr = $mp->first_child('str')->first_child('address');
            $mp_data{STR} = $str_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $str_addr->first_child_text('office');
            if ($mp_data{STR} eq '- Moving') {
              $mp_data{STR} = '[[MEPs_STR_SDF|-]]';
            }
            else {
              $mp_data{STR} =~ s/^(([A-Z][A-Z][A-Z]) [MT](\d\d).+)$/[[MEPs_$2_$3|$1]]/o;
            }

            my $role = $mp->first_child('role')->att('role');
            if ($role ne 'Member') {
              $mp_data{NAME} =~ s/]]$/ ($role)]]/o;
            }

            push @{$meps{$code}->{MEPS}}, \%mp_data;
          }
        }
	  };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

# Build mediawiki page
my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_ITRE_IMCO_anti_3_strikes|English]]''' - '''[[Eurodeputes_ITRE_IMCO_anti_riposte_graduee|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: Anti-Three-Strikes Approach ITRE & IMCO MEPs' Informations==

MEPs listed below have hit a 20/20 score by voting against three-strikes approach (graduated response) on the [[Cultural_industries_resolution|April 10th 2008 vote of a resolution on cultural industries in Europe (rapport Guy Bono)]]. ''Substiture members of ITRE & IMCO committees are shown in italics.''
MEDIAWIKI_PAGE

foreach my $group_code (sort {$groups_order{$a} <=> $groups_order{$b}} keys %meps) {
  next unless exists($meps{$group_code}->{MEPS});

  (my $group_wiki = $group_code) =~ s![/-].+$!!o;
  $mediawiki_page .= "\n===[[MEPs_$group_wiki|$meps{$group_code}->{NAME}]]===\n\n";
  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' width='90%'\n";
  $mediawiki_page .= "! MEP !! Country !! Committee !! Tel. !! Fax !! Email !! Brussels !! Strasbourg\n";
  $mediawiki_page .= "|-\n";
  foreach my $deputy (@{$meps{$group_code}->{MEPS}}) {
    $mediawiki_page .= "| $deputy->{NAME} || [[Eurodeputes_$deputy->{COUNTRY}|$deputy->{COUNTRY}]] || $deputy->{COM} || $deputy->{TEL} || $deputy->{FAX} || $deputy->{EMAIL} || $deputy->{BXL} || $deputy->{STR}\n";
    $mediawiki_page .= "|-\n";
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
}

$mediawiki_page .= "\n\n----\n\nBack to home page of [[Political_Memory|Political Memory]]\n";
$mediawiki_page .= "[[Category_Telecoms_Package]] [[Category:MEPs]] [[Category:Political_Memory]]\n";

# Output mediawiki page
my $mediawiki_location = "$output_dir/MEPs_ITRE_IMCO_anti_3_strikes.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";
