#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use LWP::UserAgent;
use XML::Twig;
use Unicode::String qw(utf8 latin1);
use File::Basename;
use locale;
use POSIX 'locale_h';
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $output_dir = "$root_dir/xml/meps";
my $debug = 0;
my $this_mep;
my $this_country;
my $this_id;

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'mep|m=s'        => \$this_mep,
  'country|c=s'    => \$this_country,
  'idmep|i=i'      => \$this_id,
  'mep|m=s'        => \$this_mep,
  'output|o=s'     => \$output_dir,
  'debug|d'        => \$debug,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-hd] [-m <MemberOfEuropeanParliamentWikiName>] [-o <output_dir]
Options can be:
  --mep, -m <MepWikiName>:           wiki name of the MEP to processed
                                     default: unset => all MEPs are processed
  --country, -c <country_code>:      country code (2 letters) of the MEPs to processed
                                     default: unset => all MEPs from all country are processed
  --idmep, -i <id_mep>:              Official identifier at European Parlement of the MEP to processed
                                     default: unset => all MEPs are processed
  --output, -o <output_dir>:         output directory of xml pages
                                     default: $root_dir/xml/meps
  --debug, -d                        debug
                                     default: no debug
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);

my %groups = (
  'Groupe du Parti populaire européen (Démocrates-chrétiens) et des Démocrates européens' => 'PPE-DE',
  'Groupe socialiste au Parlement européen' => 'PSE',
  'Groupe Alliance des démocrates et des libéraux pour l\'Europe' => 'ALDE',
  'Groupe Union pour l\'Europe des Nations' => 'UEN',
  'Groupe des Verts/Alliance libre européenne' => 'Verts/ALE',
  'Groupe confédéral de la Gauche unitaire européenne/Gauche verte nordique' => 'GUE/NGL',
  'Groupe Indépendance/Démocratie' => 'IND/DEM',
  'Groupe Identité, Tradition, Souveraineté' => 'ITS',
  'Non-inscrits' => 'NI',
);

my %buildings = (
  'Altiero Spinelli' => 'ASP',
  'Paul-Henri Spaak' => 'PHS',
  'Atrium' => 'ATR',
  'Willy Brandt' => 'WIB',
  'Louise Weiss' => 'LOW',
  'Winston Churchill' => 'WIC',
  'Salvador de Madariaga' => 'SDM',
  'En cours de déménagement' => '-',
);

my %coms = (
  'Commission des affaires étrangères' => 'AFET',
  'Sous-commission "droits de l\'homme"' => 'DROI',
  'Sous-commission "sécurité et défense"' => 'SEDE',
  'Commission du développement' => 'DEVE',
  'Commission du commerce international' => 'INTA',
  'Commission des budgets' => 'BUDG',
  'Commission du contrôle budgétaire' => 'CONT',
  'Commission des affaires économiques et monétaires' => 'ECON',
  'Commission de l\'emploi et des affaires sociales' => 'EMPL',
  'Commission de l\'environnement, de la santé publique et de la sécurité alimentaire' => 'ENVI',
  'Commission de l\'industrie, de la recherche et de l\'énergie' => 'ITRE',
  'Commission du marché intérieur et de la protection des consommateurs' => 'IMCO',
  'Commission des transports et du tourisme' => 'TRAN',
  'Commission du développement régional' => 'REGI',
  'Commission de l\'agriculture et du développement rural' => 'AGRI',
  'Commission de la pêche' => 'PECH',
  'Commission de la culture et de l\'éducation' => 'CULT',
  'Commission des affaires juridiques' => 'JURI',
  'Commission des libertés civiles, de la justice et des affaires intérieures' => 'LIBE',
  'Commission des affaires constitutionnelles' => 'AFCO',
  'Commission des droits de la femme et de l\'égalité des genres' => 'FEMM',
  'Commission des pétitions' => 'PETI',
  'Commission temporaire sur le changement climatique' => 'CLIM',
);

# Create a user agent object
my $ua = LWP::UserAgent->new;
$ua->agent("$0 (gibus perl script to fetch information for http://www.laquadrature.net/wiki/)");
$ua->env_proxy;
my $headers = HTTP::Headers->new('Accept-Language' => 'fr');

# Fetch list of countries
my $countries_url = 'http://www.europarl.europa.eu/members/expert.do?language=FR';

# Create a request
my $countries_req = HTTP::Request->new(GET => $countries_url, $headers);

# Pass request to the user agent and get a response back
warn "Fetching list of countries $countries_url\n";
my $countries_res = $ua->request($countries_req);

# Check the outcome of the response
unless ($countries_res->is_success) {
  die "Error fetching list of countries: $countries_url: ", $countries_res->status_line, "\n";
}

# Build list of countries
my %countries;
foreach my $country_line (split /\n/, $countries_res->content) {
  next unless $country_line =~ /^\s*<a href="\/members\/expert\/groupAndCountry\/search\.do[^?]*\?country=([A-Z][A-Z])&amp;language=FR" style="[^"]+" id="\1" onmouseover="overlib\('([^']+)'[^)]+\)" onmouseout="nd\(\);">/o;

  $countries{$1} = $2;
}

# Parse each country
foreach my $country_code (sort keys %countries) {
  next unless not defined $this_country or $this_country eq $country_code;
  my $country_name = $countries{$country_code};
  $country_name =~ s/&nbsp;/ /go;
  $country_name =~ s/&#8209;/-/go;

  # Fetch first page
  my $country_url = "http://www.europarl.europa.eu/members/expert/groupAndCountry/search.do?country=$country_code&language=FR";
  
  # Create a request
  my $country_req = HTTP::Request->new(GET => $country_url, $headers);
  
  # Pass request to the user agent and get a response back
  warn "Fetching first page of country $country_url\n";
  my $country_res = $ua->request($country_req);
  
  # Check the outcome of the response
  unless ($country_res->is_success) {
    die "Error fetching first page of country: $country_url: ", $country_res->status_line, "\n";
  }
  
  # Repeat for all pages
  my $country_html = $country_res->content;
  do {
    my @meps = $country_html =~ /\/members\/expert\/groupAndCountry\/view\.do(?:;jsessionid=\w+\.node\d)?\?partNumber=\d&amp;country=$country_code&amp;language=FR&amp;id=(\d+)/g;
    # Parse each MEP
    foreach my $mep_id (@meps) {
      next unless not defined $this_id or $this_id eq $mep_id;
      # Fetch MEP
      my $mep_url = "http://www.europarl.europa.eu/members/expert/alphaOrder/view.do?language=FR&id=$mep_id";
      warn "Fetching MEP $mep_url\n";
      my $mep_req = HTTP::Request->new(GET => $mep_url, $headers);
      my $mep_res = $ua->request($mep_req);
      unless ($mep_res->is_success) {
        die "Error fetching MEP: $mep_url: ", $mep_res->status_line, "\n";
      }
      my $mep_html = $mep_res->content;
  
      # Extract infos
      # Name
      my ($name_html) = ($mep_html =~ /<td colspan="3" class="mepname">(.+?)<\/td>/os);
      my ($mep_title_name, $mep_first_name, $mep_last_name) = ($name_html =~ /^\n\s+(.+?)\n\s+(.+?)\n\s+(.+?)\n\s+$/os);
      $mep_last_name = capitalize($mep_last_name);
      my $mep_name;
      $mep_title_name =~ s/^\s+//o;
      $mep_title_name =~ s/\s+$//o;
      if ($mep_title_name and $mep_title_name eq 'Graf') {
        $mep_first_name .= " $mep_title_name";
        undef $mep_title_name;
        $mep_name = "$mep_first_name $mep_last_name";
      }
      if ($mep_title_name) {
        if ($mep_first_name =~ /^\s*$/ and $mep_last_name =~ /^Nicholson Of Winterbourne$/i) {
          $mep_first_name = 'Emma';
        }
        $mep_name = "$mep_title_name $mep_first_name $mep_last_name";
      }
      else {
        $mep_name = "$mep_first_name $mep_last_name";
      }
      my $mep_wiki_name = wikify($mep_name);
      next if (defined $this_mep and $mep_wiki_name ne $this_mep);
      (my $full_name = $name_html) =~ s/\s+/ /go;
      $full_name =~ s/^\s+//o;
      $full_name =~ s/\s+$//o;
  
      # Picture
      my $mep_pic_url = "http://www.europarl.europa.eu/mepphoto/$mep_id.jpg";
      my $mep_pic_location = "$root_dir/IMG/meps/$mep_wiki_name.jpg";
      $ua->mirror($mep_pic_url, $mep_pic_location) unless $debug;
  
      # Birth
      my ($mep_birth_gender, $mep_birth_date, $mep_birth_city) = ($mep_html =~ /<!-- birth date, birth place-->\s+(Né(?:e?)) le (.+?),([^>]+)<\/td>/o);
      my ($mep_birth_day, $mep_birth_month, $mep_birth_year) = ($mep_birth_date =~ /^(\d+) (\S+) (\d+)$/o);
      $mep_birth_city =~ s/^\s+//o;
      $mep_birth_city =~ s/\s+$//o;
  
      # Political group
      my ($mep_grp, $mep_grp_role) = ($mep_html =~ /<span class="titlemep">([^<]+)<\/span><br \/>\s+<span class="titlemep2">\s+(.+?)\s*?\n/o);
      die "Unknown group <$mep_grp> for <$mep_url>\n" unless exists $groups{$mep_grp};
      my $mep_grp_abbrev = $groups{$mep_grp};
      (my $mep_party) = ($mep_html =~ /<!-- national party -->\s+([^<]+)<br \/>/o);
      $mep_party =~ s/\.?\s+$//o;
      $mep_party =~ s/&(?!amp;)/&amp;/go;
      
      # Web
      (my $mep_web) = ($mep_html =~ /<td class="mepurl">\s+<a href="([^"]+)"/o);
  
      # Extract email if included in addresses
      my @mep_emails = $mep_html =~ /<a href="mailto:([^"]+)"/go;

      # Hack for emails collected by hand
      push @mep_emails, 'joao.pinheiro@europarl.europa.eu' if $mep_wiki_name eq 'JoaodeDeusPinheiro';
      push @mep_emails, 'joerg.leichtfried@europarl.europa.eu' if $mep_wiki_name eq 'JorgLeichtfried';

      # Guess mail if not published
      my $mep_guessed_email = lc(desaccent($mep_first_name)).'.'.lc(desaccent($mep_last_name)).'@europarl.europa.eu' unless scalar @mep_emails;
  
      # Address
      # Hack for typo in one MEP info
      $mep_html =~ s/<strong>Fax<\/strong> : \+32 \(0\)2 28 45778<br \/>/<strong>Tél<\/strong> : +32 (0)2 28 45778<br \/>/o;
      # Brussels Parliament
      my ($bxl_building, $bxl_office, $bxl_phone_ext) = ($mep_html =~ /<strong>Parlement&nbsp;europ&eacute;en<\/strong><br \/>\s+B&acirc;t\. ([^<]+)<br \/>\s+([^<]+)<br \/>\s+60, rue Wiertz \/ Wiertzstraat 60<br \/>\s+B-1047 <strong>Bruxelles\/Brussel<\/strong>(?:<br \/>\s+<strong>Tél<\/strong> : \+32 \(0\)2 28 4[57](\d\d\d))?(?:<br \/>\s+<strong>Tél<\/strong> : \+32 \(0\)2 28 47\3)?(?:<br \/>\s+<strong>Fax<\/strong> : \+32 \(0\)2 28 49\3<br \/>)?/o);
      
      # Strasbourg Parliament
      my ($stb_building, $stb_office, $stb_phone_ext) = ($mep_html =~ /<strong>Parlement europ&eacute;en<\/strong><br \/>\s+B&acirc;t\. ([^<]+)<br \/>\s+([^<]+)<br \/>\s+1, avenue du Président Robert Schuman<br \/>\s+CS 91024<br \/>\s+F-67070 <strong>Strasbourg Cedex<\/strong>(?:<br \/>\s+<strong>Tél<\/strong> : \+33 \(0\)3 88 1 7[57](\d\d\d))?(?:<br \/>\s+<strong>Fax<\/strong> : \+33 \(0\)3 88 1 79\3<br \/>)?/);
      
      $bxl_building = $bxl_office = 'En cours de déménagement' unless $bxl_building and $bxl_office;
      $stb_building = $stb_office = 'En cours de déménagement' unless $stb_building and $stb_office;
      my $phone_ext = $bxl_phone_ext || $stb_phone_ext || 'Non encore attribué';
      
      my @mep_address = (
        {
          LABEL=>'Parlement européen',
          STREET=>'60, rue Wiertz',
          BUILDING=>$bxl_building,
          OFFICE=>$bxl_office,
          POSTCODE=>'1047',
          CITY=>'Bruxelles',
          TEL=>["+32 2 28 45 $phone_ext", "+32 2 28 47 $phone_ext"],
          FAX=>["+32 2 28 49 $phone_ext"],
        },
        {
          LABEL=>'Parlement européen',
          STREET=>'1, avenue du Président Robert Schuman - CS 91024',
          BUILDING=>$stb_building,
          OFFICE=>$stb_office,
          POSTCODE=>'67070',
          CITY=>'Strasbourg Cedex',
          TEL=>["+33 3 88 175 $phone_ext", "+33 3 88 177 $phone_ext"],
          FAX=>["+33 3 88 179 $phone_ext"],
        }
      );
  
      # Other address
      my ($other_address) = ($mep_html =~ /<span class="txtmep">(.+?)<\/span>/os);
      if ($other_address) {
        $other_address =~ s/\s\s+//go;
        $other_address =~ s/^<strong>Adresses? postales?&nbsp;(?::&nbsp;)?<\/strong>(?:<br \/>)?//o;
        foreach my $addr (split /<br \/>/, $other_address) {
          # No need to repeat Parliament address
          next if $addr =~ /&nbsp;(?:(?:60, r|R)ue Wiertz|Wiertzstraat)&nbsp;/o;

          my @parts = reverse(split /&nbsp;/, $addr);
          my %addr;

          # UK address
          if ($country_code eq 'GB') {
            $addr{POSTCODE} = shift @parts;
            my $city_district = shift @parts;
            ($addr{CITY}, $addr{DISTRICT}) = split /, /, $city_district;
            $addr{STREET} = shift @parts;
            $addr{LABEL} = shift @parts if scalar @parts;

            # Fix typo
            $addr{STREET} =~ s/Suothwark/Southwark/o;
            # Dirty hacks for special cases
            if ($addr{STREET} =~ /^\([^)]+\)$/o or
                $addr{STREET} eq 'Gosforth' or
                $addr{STREET} eq 'Greenfield' or
                $addr{STREET} eq 'Euxton' or
                $addr{STREET} eq 'Wynford Square' or
                $addr{STREET} eq 'Thomas Street' or
                $addr{STREET} eq '222 Mill Road' or
                $addr{STREET} eq 'County Hall' or
                $addr{STREET} eq 'Mount Stuart Square'
               ) {
              $addr{STREET} = "$addr{LABEL} $addr{STREET}";
              delete $addr{LABEL};
            }
            elsif ($addr{STREET} eq 'Crane Mead' or
                   $addr{STREET} eq 'Southmoor' or
                   $addr{STREET} eq 'Wick' or
                   $addr{STREET} eq 'Ham Lane'
                  ) {
              $addr{STREET} = "$addr{LABEL} $addr{STREET}";
              $addr{LABEL} = shift @parts if scalar @parts;
            }
            elsif ($addr{STREET} eq 'Middlesex') {
              $addr{DISTRICT} = $addr{STREET};
              $addr{STREET} = shift @parts if scalar @parts;
              $addr{STREET} .= " $addr{LABEL}";
              delete $addr{LABEL};
            }
            elsif ($addr{CITY} eq 'Surrey') {
              $addr{DISTRICT} = $addr{CITY};
              $addr{CITY} = $addr{STREET};
              $addr{STREET} = $addr{LABEL};
              delete $addr{LABEL};
            }
            elsif ($addr{STREET} eq '24 Southwark Street' or
                   $addr{STREET} eq 'Bow Street'
                  ) {
              $addr{STREET} = shift(@parts) . " $addr{LABEL} $addr{STREET}";
              delete $addr{LABEL};
            }
          }
          # Other countries address
          else {
            $addr{CITY} = shift @parts;
            ($addr{POSTCODE} = shift @parts) =~ s/^[A-Z][A-Z]-//o;
            $addr{STREET} = shift @parts;
            unless ($addr{STREET}) {
              $addr{STREET} = $addr{POSTCODE};
              undef $addr{POSTCODE};
            }
            $addr{LABEL} = shift @parts if scalar @parts;

            # Dirty hacks for special cases
            if ($addr{CITY} eq 'Cedex') {
              $addr{CITY} = "$addr{POSTCODE} $addr{CITY}";
              ($addr{POSTCODE} = $addr{STREET}) =~ s/^[A-Z][A-Z]-//o;
              $addr{STREET} = $addr{LABEL};
              $addr{LABEL} = shift @parts if scalar @parts;
            }
            elsif ($addr{CITY} eq '(Malta)') {
              $addr{CITY} = $addr{POSTCODE};
              delete $addr{POSTCODE};
            }

            if ($addr{STREET} eq '&quot;Caphan&quot;' or
                $addr{STREET} eq 'Le Clos de l&#39;Aireau' or
                $addr{STREET} =~ /^BP /o or
                $addr{STREET} =~ /^P\.?O\.? BOX/i or
                $addr{STREET} eq 'Triq Fomm il-Ghelliem' or
                $addr{STREET} =~ /^Sector \d+$/ or
                $addr{STREET} eq 'bl. D7, Sc. 1, ap. 24' or
                $addr{STREET} eq 'sc. A, Ap. 2' or
                $addr{STREET} eq 'Unter den Linden 50' or
                $addr{STREET} eq 'Dresdner Hof' or
                $addr{STREET} eq 'Puistokatu 34 A' or
                $addr{STREET} eq 'Rue du Curat' or
                $addr{STREET} eq 'Lg Jean Monnet, nº1 - 6º andar' or
                $addr{STREET} eq 'Nr. 3-5, Sector 4'
               ) {
              $addr{STREET} = "$addr{LABEL} $addr{STREET}" if $addr{LABEL};
              delete $addr{LABEL};
            }
            elsif ($addr{POSTCODE} and (
                   $addr{POSTCODE} eq 'Douglas' or
                   $addr{POSTCODE} eq 'Union Street' or
                   $addr{POSTCODE} eq 'Drumconrath' or
                   $addr{POSTCODE} eq 'Ballinabearna'
                  )) {
              $addr{STREET} .= " $addr{POSTCODE}";
              delete $addr{POSTCODE};
            }
            elsif ($addr{POSTCODE} and $addr{POSTCODE} eq 'DUBLIN') {
              $addr{CITY} = "$addr{POSTCODE} $addr{CITY}";
              $addr{STREET} = "$addr{LABEL} $addr{STREET}" if $addr{LABEL};
              delete $addr{LABEL};
              delete $addr{POSTCODE};
            }

            if ($addr{STREET} eq 'Office 103 P.O. Box 21857, 1514' or
                $addr{STREET} eq 'Bâtiment Ivoire Rue du Curat' or
                $addr{STREET} eq 'Bloc 2, sc. A, apt. 15 Sector 1'
               ) {
              $addr{STREET} = shift(@parts) . " $addr{STREET}";
            }
          }

          if (scalar @parts) {
            if ($addr{LABEL}) {
              $addr{LABEL} = join(' ', @parts) . " $addr{LABEL}";
            }
            else {
              $addr{LABEL} = join ' ', @parts;
            }
          }

          $addr{CITY} = capitalize($addr{CITY});
          $addr{CITY} =~ s/(.)&#39;/\l$1'/go;
          $addr{STREET} =~ s/(.)&#39;/\l$1'/go;
          $addr{STREET} =~ s/&quot;/"/go;
          $addr{LABEL} =~ s/&#39;/'/go if $addr{LABEL};

          push @mep_address, \%addr;
        }
      }
  
      # CV
      my ($mep_cv_txt) = ($mep_html =~ /<table[^>]+>\s*<tr>\s*<td[^>]+>Curriculum Vitae<\/td>\s*<\/tr>\s*(.+?)<tr class="mep_CVtext"><td><br\/><\/td><\/tr>\s*<\/table>/os);
      $mep_cv_txt = "" unless $mep_cv_txt;
      $mep_cv_txt =~ s/\s\s+//go;
      $mep_cv_txt =~ s/<td[^>]+"><img[^>]+><\/td>/ /go;
      $mep_cv_txt =~ s/<\/?t[rd][^>]*>//go;
      $mep_cv_txt =~ s/&#39;/'/go;
      $mep_cv_txt =~ s/&quot;/'/go;
      $mep_cv_txt =~ s/^ //go;
      $mep_cv_txt =~ s/\.$//go;
      $mep_cv_txt =~ s/&amp;/&/go;
      my @mep_cv_lines = map {$_=ucfirst $_; s/&/&amp;/g; "$_."} split /\. (?!\d\d\d\d)| ?; /o, $mep_cv_txt;

      # Functions
      my ($mep_functions_txt) = ($mep_html =~ /<!-- END : MEP ID CARD -->\s*<br\/>\s*<table[^>]+>\s*(?:<tr><td>\s*<\/td><\/tr>\s*)*<tr><td>\s*<table[^>]+>\s*<tr>\s*(.+?)\s*<tr class="mep_CVtext"><td><br\/><\/td><\/tr>\s*<\/table>\s*(?:<\/td><\/tr>\s*<tr><td>\s*)*(?:<!-- START : PARLIAMENTARY WORKS  -->|<table[^>]+>\s*<tr>\s*<td[^>]+>Curriculum [Vv]itae<\/td>)/os);
      $mep_functions_txt = "" unless $mep_functions_txt;
      $mep_functions_txt =~ s/\s\s+//go;
      $mep_functions_txt =~ s/\n//go;
      $mep_functions_txt =~ s/<\/?tr[^>]*>//go;
      $mep_functions_txt =~ s/<\/?table[^>]*>//go;
      $mep_functions_txt =~ s/<a[^>]+>[^<]+<\/a>//go;
      $mep_functions_txt =~ s/<td><br\/><\/td>//go;
      $mep_functions_txt =~ s/<td[^>]+"><img[^>]+><\/td>/ /go;
      $mep_functions_txt =~ s/<\/?td>//go;
      $mep_functions_txt =~ s/ <td/<td/go;
      my @mep_functions_by_role = split /<td colspan="2" class="mepcountry">/o, $mep_functions_txt;
      shift @mep_functions_by_role;
      my @mep_functions;
      foreach my $func (@mep_functions_by_role) {
        $func =~ s/^(.+?)(?=<td)//o;
        my $role = $1;
        my @func = split /<td style="width: 98%;">/o, $func;
        shift @func;
        map {push @mep_functions, {ROLE=>$role, VALUE=>$_}} @func;
      }

      # Activities
      my $questions_url = "http://www.europarl.europa.eu/sidesSearch/search.do?type=QP&amp;language=FR&amp;term=6&amp;author=$mep_id";
      my $motions_url = "http://www.europarl.europa.eu/sidesSearch/search.do?type=MOTION&amp;language=FR&amp;term=6&amp;author=$mep_id";
      my $reports_url = "http://www.europarl.europa.eu/sidesSearch/search.do?type=REPORT&amp;language=FR&amp;term=6&amp;author=$mep_id";
      my $opinions_url = "http://www.europarl.europa.eu/sidesSearch/search.do?type=COMPARL&amp;subType=OPCF&amp;language=FR&amp;term=6&amp;author=$mep_id";
      my $debates_url = "http://www.europarl.europa.eu/sidesSearch/search.do?type=CRE&amp;language=FR&amp;term=6&amp;author=$mep_id";
      my $declarations_url = "http://www.europarl.europa.eu/sidesSearch/search.do?type=WDECL&amp;language=FR&amp;term=6&amp;author=$mep_id";
  
      # Build XML tree
      my $xml = XML::Twig->new(pretty_print => 'indented', output_encoding=>'UTF-8');
      $xml->set_encoding('UTF-8');
      my $root = XML::Twig::Elt->new('politician');
      $xml->set_root($root);
      $root->insert_new_elt('first_child', 'container', 'meps.dbxml');
      my $info = $root->insert_new_elt('last_child', 'infos');
      my $name = $info->insert_new_elt('last_child', 'name');
      $name->insert_new_elt('last_child', 'title', $mep_title_name) if $mep_title_name;
      $name->insert_new_elt('last_child', 'first', $mep_first_name);
      $name->insert_new_elt('last_child', 'last', $mep_last_name);
      $name->insert_new_elt('last_child', 'full', $full_name);
      $name->insert_new_elt('last_child', 'wiki', $mep_wiki_name);
      $info->insert_new_elt('last_child', 'picture', basename($mep_pic_location));
      my $birth = $info->insert_new_elt('last_child', 'birth');
      $birth->insert_new_elt('last_child', 'gender', $mep_birth_gender);
      my $birth_date = $birth->insert_new_elt('last_child', 'date');
      $birth_date->insert_new_elt('last_child', 'day', $mep_birth_day);
      $birth_date->insert_new_elt('last_child', 'month', $mep_birth_month);
      $birth_date->insert_new_elt('last_child', 'year', $mep_birth_year);
      my $birth_place = $birth->insert_new_elt('last_child', 'place');
      $birth_place->insert_new_elt('last_child', 'city', $mep_birth_city);
      my $constituency = $info->insert_new_elt('last_child', 'constituency');
      my $country = $constituency->insert_new_elt('last_child', 'country');
      $country->insert_new_elt('last_child', 'name', $country_name);
      $country->insert_new_elt('last_child', 'code', $country_code);
      my $group = $info->insert_new_elt('last_child', 'group');
      $group->set_att('role', $mep_grp_role) if $mep_grp_role;
      $group->insert_new_elt('last_child', 'name', $mep_grp);
      $group->insert_new_elt('last_child', 'abbreviation', $mep_grp_abbrev);
      $group->insert_new_elt('last_child', 'party', $mep_party);
      my $contact = $root->insert_new_elt('last_child', 'contact');
      $mep_url =~ s/&(?!amp;)/&amp;/go;
      $contact->insert_new_elt('last_child', web => { type => "Page sur le site du Parlement européen" }, $mep_url);
      $contact->insert_new_elt('last_child', 'web', $mep_web) if $mep_web;
      foreach my $mep_email (@mep_emails) {
        $contact->insert_new_elt('last_child', 'email', $mep_email);
      }
      if ($mep_guessed_email) {
        my $guessed_email = $contact->insert_new_elt('last_child', 'email', $mep_guessed_email);
        $guessed_email->set_att('guessed', 'true');
      }
      foreach my $addr (@mep_address) {
        my $address = $contact->insert_new_elt('last_child', 'address');
        $address->insert_new_elt('last_child', 'label', $addr->{LABEL}) if $addr->{LABEL};
        if ($addr->{BUILDING}) {
          my $building = $address->insert_new_elt('last_child', 'building');
          $building->insert_new_elt('last_child', 'name', $addr->{BUILDING});
          my $building_abbrev = $buildings{$addr->{BUILDING}};
          $building->insert_new_elt('last_child', 'abbreviation', $building_abbrev) if $building_abbrev;
        }
        $address->insert_new_elt('last_child', 'office', $addr->{OFFICE}) if $addr->{OFFICE};
        $address->insert_new_elt('last_child', 'street', $addr->{STREET}) if $addr->{STREET};
        $address->insert_new_elt('last_child', 'postcode', $addr->{POSTCODE}) if $addr->{POSTCODE};
        $address->insert_new_elt('last_child', 'city', $addr->{CITY}) if $addr->{CITY};
        $address->insert_new_elt('last_child', 'district', $addr->{DISTRICT}) if $addr->{DISTRICT};
        if ($addr->{TEL}) {
          map {$address->insert_new_elt('last_child', 'phone', $_)} @{$addr->{TEL}};
        }
        if ($addr->{FAX}) {
          map {$address->insert_new_elt('last_child', 'fax', $_)} @{$addr->{FAX}};
        }
      }
      my $functions = $root->insert_new_elt('last_child', 'functions');
      foreach my $func (@mep_functions) {
        my $function = $functions->insert_new_elt('last_child', 'function');
        $function->insert_new_elt('last_child', 'label', $func->{VALUE});
        $function->insert_new_elt('last_child', 'abbreviation', $coms{$func->{VALUE}}) if $coms{$func->{VALUE}};
        $function->insert_new_elt('last_child', 'role', $func->{ROLE});
      }
      my $cv = $root->insert_new_elt('last_child', 'cv');
      foreach my $pos (@mep_cv_lines) {
        $cv->insert_new_elt('last_child', 'position', $pos);
      }
      my $activities = $root->insert_new_elt('last_child', 'activities');
      $activities->insert_new_elt('last_child', 'questions', $questions_url);
      $activities->insert_new_elt('last_child', 'motions', $motions_url);
      $activities->insert_new_elt('last_child', 'reports', $reports_url);
      $activities->insert_new_elt('last_child', 'opinions', $opinions_url);
      $activities->insert_new_elt('last_child', 'debates', $debates_url);
      $activities->insert_new_elt('last_child', 'declarations', $declarations_url);
  
      my $xml_location = "$output_dir/$mep_wiki_name.xml";
      if (-f $xml_location) {
        my $old_twig = XML::Twig->new(keep_encoding=>1)->parsefile($xml_location);
        $old_twig->root->first_child('scores')->copy->paste('last_child', $root)
          if $old_twig->root->first_child('scores');
        $old_twig->root->first_child('opinions')->copy->paste('last_child', $root)
          if $old_twig->root->first_child('opinions');
      }

      open XML, ">$xml_location" or die "Cannot write in file $xml_location: $!\n";
      $xml->flush(\*XML);
 	    close XML or die "Error when closing $xml_location: $!\n";
  
      warn "first=!$mep_first_name! last=!$mep_last_name! wiki=!$mep_wiki_name! day=!$mep_birth_day! month=!$mep_birth_month! year=!$mep_birth_year! city=!$mep_birth_city! role=!$mep_grp_role! grp=!$mep_grp! abbrev=!$mep_grp_abbrev! party=!$mep_party!".($mep_web?" web=!$mep_web!":'').(scalar @mep_emails?join('/', map {" email=!$_!"} @mep_emails):'')."\naddr=[ ".join(" ]\naddr=[ ", map({($_->{LABEL}?"label=!$_->{LABEL}! ":'')."street=!$_->{STREET}! postcode=!$_->{POSTCODE}! city=!$_->{CITY}! ".($_->{DISTRICT}?"district=!$_->{DISTRICT}! ":'').($_->{TEL}?"tel=!".join(" /",@{$_->{TEL}})."!":'').($_->{FAX}?"fax=!".join("/",@{$_->{FAX}})."!":'')} @mep_address))."]\ncv=[ ".join(" ]\ncv=[ ", map({$_} @mep_cv_lines))." ]\nfunction=[ ".join(" ]\nfunction=[ ", map({$_->{VALUE}." (".$_->{ROLE}.")"} @mep_functions))." ]\n\n" if $debug;
    }

    if ($country_html =~ /&amp;(partNumber=2)/) {
      $country_url = "http://www.europarl.eu.int/members/expert/groupAndCountry/search.do?country=$country_code&language=FR&$1";
      # Fetch second page
      my $second_req = HTTP::Request->new(GET => $country_url, $headers);
  
      # Pass request to the user agent and get a response back
      warn "Fetching second page of country $country_url\n";
      my $second_res = $ua->request($second_req);
  
      # Check the outcome of the response
      unless ($second_res->is_success) {
        die "Error fetching second page of country: $country_url: ", $second_res->status_line, "\n";
      }
  
      # Repeat for all pages
      $country_html = $second_res->content;
    }
    else {
      undef $country_html;
    }
  } while ($country_html);
}

sub capitalize {
  my $str = shift; 
  my $original_len = length $str;
  # Hack since accented letters cannot be changed in lower/upper case in utf8
  setlocale(LC_CTYPE, "fr_FR");
  $str = utf8($str)->latin1;
  $str =~ s/(\w+)/\u\L$1/go;
  $str = latin1($str)->utf8;
  setlocale(LC_CTYPE, "fr_FR.UTF-8");
  # Particule should be in lower case
  $str =~ s/\bD([e'])\b/d$1/go;

  # Dirty hacks for non-latin1 letter
  $str =~ s/^Boboíková$/Bobošíková/o;
  $str =~ s/^Bezina$/Březina/o;
  $str =~ s/^Ducho$/Duchoň/o;
  #$str =~ s/Věra/Vera/o; #28336
  $str =~ s/^Hybáková$/Hybášková/o;
  $str =~ s/^Kohlíek$/Kohlíček/o;
  $str =~ s/^Matálka$/Maštálka/o;
  $str =~ s/^Rouek$/Rouček/o;
  $str =~ s/^Kottová$/Škottová/o;
  $str =~ s/^Strejek$/Strejček/o;
  $str =~ s/^Stro$/Strož/o;
  $str =~ s/^Elezný$/Železný/o;
  $str =~ s/^Zvina$/Zvěřina/o;
  $str =~ s/^Horáek$/Horáček/o;
  $str =~ s/^Ry$/Őry/o;
  $str =~ s/^Andrikien$/Andrikienė/o;
  $str =~ s/^Budreikait$/Budreikaitė/o;
  $str =~ s/^Dikut$/Dičkutė/o;
  $str =~ s/^Didiokas$/Didžiokas/o;
  $str =~ s/^Jukneviien$/Juknevičienė/o;
  $str =~ s/^Starkeviit$/Starkevičiūtė/o;
  $str =~ s/^Kuis$/Kušķis/o;
  $str =~ s/^Pks$/Pīks/o;
  $str =~ s/^Danoka$/Ždanoka/o;
  $str =~ s/^Zle$/Zīle/o;
  $str =~ s/^Hoowczyc$/Hołowczyc/o;
  $str =~ s/^Jaowiecki$/Jałowiecki/o;
  $str =~ s/^Kuakowski$/Kułakowski/o;
  $str =~ s/^Kumiuk$/Kuźmiuk/o;
  $str =~ s/^Pk$/Pęk/o;
  $str =~ s/^Podkaski$/Podkański/o;
  $str =~ s/^Szymaski$/Szymański/o;
  $str =~ s/^Zapaowski$/Zapałowski/o;
  $str =~ s/^Botinaru$/Boştinaru/o;
  $str =~ s/^Corlean$/Corlăţean/o;
  $str =~ s/^Creu$/Creţu/o;
  $str =~ s/^Dianu$/Dăianu/o;
  $str =~ s/^Pacu$/Paşcu/o;
  $str =~ s/^Icu$/Ţicău/o;
  $str =~ s/^Tkés$/Tőkés/o;
  $str =~ s/^Vlean$/Vălean/o;
  $str =~ s/^Drar Murko$/Drčar Murko/o;
  $str =~ s/^Buoi$/Buşoi/o;
  $str =~ s/^Frunzverde$/Frunzăverde/o;
  $str =~ s/^Mnescu$/Mănescu/o;
  $str =~ s/^Beová$/Beňová/o;
  $str =~ s/^Gaa$/Gaľa/o;
  $str =~ s/^Maka$/Maňka/o;
  $str =~ s/^Mikoláik$/Mikolášik/o;
  $str =~ s/^Pletinská$/Pleštinská/o;
  $str =~ s/^Astný$/Šťastný/o;

  $str =~ s/^Praha - Libu$/Praha - Libuš/o;
  $str =~ s/^Rga$/Rīga/o;
  $str =~ s/^Beri$/Berği/o;
  $str =~ s/^Wrocaw$/Wrocław/o;
  $str =~ s/^Lód$/Łódź/o;
  $str =~ s/^Ód$/Łódź/o;
  $str =~ s/^Pozna$/Poznań/o;
  $str =~ s/^Wrocaw$/Wrocław/o;
  $str =~ s/^Bielsko-Biaa$/Bielsko-Biała/o;
  $str =~ s/^Kpno$/Kępno/o;
  $str =~ s/^Przemyl$/Przemyśl/o;
  $str =~ s/^Ploieti$/Ploieşti/o;
  $str =~ s/^Bucureti$/Bucureşti/o;
  $str =~ s/^Dumbrvia, Jud\. Timi$/Dumbrăviţa, Jud. Timiş/o;
  $str =~ s/^Timioara$/Timişoara/o;
  $str =~ s/^Iai$/Iaşi/o;
  $str =~ s/^Braov$/Braşov/o;
  $str =~ s/^Galai$/Galați/o;
  $str =~ s/^Dob Pri Domalah$/Dob Pri Domžalah/o;
  $str =~ s/^Domale$/Domžale/o;
  $str =~ s/^Petroani$/Petroşani/o;
  $str =~ s/^Empeter Pri Gorici$/Šempeter Pri Gorici/o;
  $str =~ s/^Preov$/Prešov/o;
  $str =~ s/^Stará Ubova$/Stará Ľubovňa/o;

  my $final_len = length $str;
  return $str;
}

sub wikify {
  my $str = shift;
  # Delete nickname
  $str =~ s/ \([^\)]+\)//o;
  $str = desaccent($str);
  $str =~ s/-//go;
  return $str;
}

sub desaccent {
  my $str = shift;
  $str =~ s/Á/A/go;
  $str =~ s/À/A/go;
  $str =~ s/Â/A/go;
  $str =~ s/Ä/A/go;
  $str =~ s/Å/A/go;
  $str =~ s/Ă/A/go;
  $str =~ s/Ã/A/go;
  $str =~ s/É/E/go;
  $str =~ s/È/E/go;
  $str =~ s/Ê/E/go;
  $str =~ s/Ë/E/go;
  $str =~ s/Ě/E/go;
  $str =~ s/Ė/E/go;
  $str =~ s/Ę/E/go;
  $str =~ s/Í/I/go;
  $str =~ s/Î/I/go;
  $str =~ s/Ï/I/go;
  $str =~ s/Ī/I/go;
  $str =~ s/Ó/O/go;
  $str =~ s/Ô/O/go;
  $str =~ s/Ö/O/go;
  $str =~ s/Ø/O/go;
  $str =~ s/Ő/O/go;
  $str =~ s/Ù/U/go;
  $str =~ s/Û/U/go;
  $str =~ s/Ü/U/go;
  $str =~ s/Ū/U/go;
  $str =~ s/Ý/Y/go;
  $str =~ s/Ç/C/go;
  $str =~ s/Ľ/L/go;
  $str =~ s/Ł/L/go;
  $str =~ s/Č/C/go;
  $str =~ s/Ģ/G/go;
  $str =~ s/Ķ/K/go;
  $str =~ s/Ñ/n/go;
  $str =~ s/Ň/N/go;
  $str =~ s/Ń/N/go;
  $str =~ s/Ř/r/go;
  $str =~ s/Š/S/go;
  $str =~ s/Ş/S/go;
  $str =~ s/Ś/S/go;
  $str =~ s/Ţ/T/go;
  $str =~ s/Ť/T/go;
  $str =~ s/Ț/T/go;
  $str =~ s/Ż/Z/go;
  $str =~ s/Ź/Z/go;
  $str =~ s/Ž/Z/go;
  $str =~ s/á/a/go;
  $str =~ s/à/a/go;
  $str =~ s/â/a/go;
  $str =~ s/ä/a/go;
  $str =~ s/å/a/go;
  $str =~ s/ă/a/go;
  $str =~ s/ã/a/go;
  $str =~ s/é/e/go;
  $str =~ s/è/e/go;
  $str =~ s/ê/e/go;
  $str =~ s/ë/e/go;
  $str =~ s/ě/e/go;
  $str =~ s/ė/e/go;
  $str =~ s/ę/e/go;
  $str =~ s/í/i/go;
  $str =~ s/î/i/go;
  $str =~ s/ï/i/go;
  $str =~ s/ī/i/go;
  $str =~ s/ó/o/go;
  $str =~ s/ô/o/go;
  $str =~ s/ö/o/go;
  $str =~ s/ø/o/go;
  $str =~ s/ő/o/go;
  $str =~ s/ú/u/go;
  $str =~ s/ù/u/go;
  $str =~ s/û/u/go;
  $str =~ s/ü/u/go;
  $str =~ s/ū/u/go;
  $str =~ s/ý/y/go;
  $str =~ s/ç/c/go;
  $str =~ s/č/c/go;
  $str =~ s/ģ/g/go;
  $str =~ s/ķ/k/go;
  $str =~ s/ľ/l/go;
  $str =~ s/ł/l/go;
  $str =~ s/ñ/n/go;
  $str =~ s/ň/n/go;
  $str =~ s/ń/n/go;
  $str =~ s/ř/r/go;
  $str =~ s/š/s/go;
  $str =~ s/ş/s/go;
  $str =~ s/ś/s/go;
  $str =~ s/ţ/t/go;
  $str =~ s/ť/t/go;
  $str =~ s/ț/t/go;
  $str =~ s/ż/z/go;
  $str =~ s/ź/z/go;
  $str =~ s/ž/z/go;
  $str =~ s/[ '.]//go;
  return $str;
}
