#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use Sleepycat::DbXml 'simple';
use IO::File ;
use File::Basename;
use Getopt::Long;

# Some files are related to the executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'meps.dbxml';

# Used for metadata
my $mdURI = 'http://www.laquadrature.net/wiki/';
my $mdName = 'timeStamp';

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] <xml_file>[, <xml_file>]*
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> can be meps, en_meps, votes, en_votes or mps
                                     (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: meps.dbxml
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(\d*)(?:\.dbxml)?$/s$1/;

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv,
            Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG, 0);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml", Db::DB_CREATE);

# Iterate through the list of files to add, read each into a string and 
#   put that string into the now opened container.
my $total = 0;
eval {
  foreach my $theFile (@ARGV)
  {
      #  Get the document name. this strips off any path and suffix information.
      (my $theName,undef,undef) = fileparse($theFile, qr/\.[^.]*/);

      #  Load the xml document into a string 
      my $xmlString;
      my $inFile = new IO::File "<$theFile"
                    or die "Cannot open $theFile: $!\n" ;
      while (<$inFile>)
      {
          $xmlString .= $_ ;
      }          

      $inFile->close();

      # Find if document already exists
      my $xmlDoc;
      eval {
        $xmlDoc = $container->getDocument($theName);
      };

      my $op;
      if ($xmlDoc) {
        # Delete the document
        $container->deleteDocument($xmlDoc);
        $op = 'Update';
      }
      else {
        # Create the document
        $xmlDoc = $theMgr->createDocument();
        $op = 'Add';
      }

      # Set the document name
      $xmlDoc->setName( $theName );
      #  Set the xml document's content to be the xmlString we just obtained. 
      $xmlDoc->setContent( $xmlString );

      # Set metadata who's value is the current date and time.
      # Get the local time
      my $timeString = time ;
      # Set the localtime onto the timeStamp metadata attribute
      $xmlDoc->setMetaData( $mdURI, $mdName, $timeString );
      # Place that document into the container 
      $container->putDocument($xmlDoc);

      warn "\t$op $theFile to container $theContainer.\n" ;
      $total++;

  } # end files2add iterator
} ;

if (my $e = catch std::exception)
{
    warn "Error adding XML data to container $theContainer\n" ;
    warn $e->what() . "\n";
    exit( -1 );
}
elsif ($@)
{
    warn "Error adding XML data to container $theContainer\n" ;
    warn $@;
    exit( -1 );
}

warn"$total file".($total>1?'s':'')." added\n";
exit 0 ;
