#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'meps.dbxml';
my $output_dir = "$root_dir/mediawiki/meps_committees";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: meps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/meps_committees
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

my %committee_meps;
my %committee_country_meps;

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query = <<"QUERY";
for \$committee in distinct-values(collection("dbxml:/$theContainer.dbxml")/politician/functions/function/abbreviation)
order by \$committee
return
  <result>
    <code>{\$committee}</code>
    {
    for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[functions/function/abbreviation/string()=\$committee]
    order by upper-case(\$deputy/infos/name/last/string()), upper-case(\$deputy/infos/name/first/string())
    return
      <deputy>
        <committee>{ \$deputy/functions/function[abbreviation/string()=\$committee]/label/string() }</committee>
        <role>{ \$deputy/functions/function[abbreviation/string()=\$committee]/role/string() }</role>
        <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "/fr|", \$deputy/infos/name/full/string(), "]]") }</name>
        <country>{ concat("{{#icon:", \$deputy/infos/constituency/country/code/string(), ".png|", \$deputy/infos/constituency/country/name/string(), "||Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "}} [[Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "|", \$deputy/infos/constituency/country/name/string(), "]]") }</country>
        <country_name>{ \$deputy/infos/constituency/country/name/string() }</country_name>
        <country_code>{ \$deputy/infos/constituency/country/code/string() }</country_code>
        <grp>{ \$deputy/infos/group/abbreviation/string() }</grp>
        <tel>{ string-join(for \$tel in \$deputy/contact//phone return concat("Tél : [callto://", replace(\$tel/string(), " ", ""), " ", \$tel/string(), "]"), "<br />") }</tel>
        <fax>{ string-join(for \$fax in \$deputy/contact//fax return concat("Fax : ", \$fax/string()), "<br />") }</fax>
        <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "@", "(à)"), "]"), "<br />") }</email>
        <bxl>{ \$deputy/contact/address[label="Parlement européen"][city="Bruxelles"] }</bxl>
        <str>{ \$deputy/contact/address[label="Parlement européen"][city="Strasbourg Cedex"] }</str>
      </deputy>
    }
  </result>
QUERY
    #  Perform the query
    eval {
        my $results = $theMgr->query($query);
        my $value;
        while( $results->next($value) ) {
          my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
          my $code =  $xml->root->first_child_text('code');
          foreach my $mp ($xml->root->children('deputy')) {
            $committee_meps{$code}->{NAME} = $mp->first_child_text('committee');
            my %mp_data;
            $mp_data{NAME} = $mp->first_child_text('name');
            $mp_data{COUNTRY} = $mp->first_child_text('country');
            $mp_data{GRP} = $mp->first_child_text('grp');
            $mp_data{TEL} = $mp->first_child_text('tel');
            $mp_data{TEL} =~ s/Tél : \[callto:\/\/\+32/Tél Bxl. : [callto:\/\/+32/go;
            $mp_data{TEL} =~ s/Tél : \[callto:\/\/\+33/Tél Str. : [callto:\/\/+33/go;
            $mp_data{FAX} = $mp->first_child_text('fax');
            $mp_data{FAX} =~ s/Fax : \+32/Fax Bxl. : +32/go;
            $mp_data{FAX} =~ s/Fax : \+33/Fax Str. : +33/go;
            $mp_data{EMAIL} = $mp->first_child_text('email');
            my $bxl_addr = $mp->first_child('bxl')->first_child('address');
            $mp_data{BXL} = $bxl_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $bxl_addr->first_child_text('office');
            if ($mp_data{BXL} eq '- En cours de déménagement') {
              $mp_data{BXL} = '[[Eurodeputes_BXL_SDF|-]]';
            }
            else {
              $mp_data{BXL} =~ s/^(([A-Z][A-Z][A-Z]) (\d\d).+)$/[[Eurodeputes_$2_$3|$1]]/o;
            }

            my $str_addr = $mp->first_child('str')->first_child('address');
            $mp_data{STR} = $str_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $str_addr->first_child_text('office');
            if ($mp_data{STR} eq '- En cours de déménagement') {
              $mp_data{STR} = '[[Eurodeputes_STR_SDF|-]]';
            }
            else {
              $mp_data{STR} =~ s/^(([A-Z][A-Z][A-Z]) [MT](\d\d).+)$/[[Eurodeputes_$2_$3|$1]]/o;
            }

            my $role = $mp->first_child_text('role');
            if ($role eq 'Membre suppléant') {
              $mp_data{NAME} =~ s/\|([^\]]+)\]/|''$1'']/;
            }
            elsif ($role ne 'Membre') {
              $mp_data{NAME} =~ s/]]$/ ($role)]]/o;
            }

            push @{$committee_meps{$code}->{MEPS}}, \%mp_data;

            my $country_code = $mp->first_child_text('country_code');
            my $country = $mp->first_child_text('country_name');
            $committee_country_meps{$country_code}->{NAME} = $country;
            $committee_country_meps{$country_code}->{COMS}->{$code}->{NAME} = $committee_meps{$code}->{NAME};
            push @{$committee_country_meps{$country_code}->{COMS}->{$code}->{MEPS}}, \%mp_data;
          }
        }
	  };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

# Build mediawiki page for navigation
my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_by_committee|English]]''' - '''[[Eurodeputes_par_commission|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : coordonnées des eurodéputés par commission parlementaire==

MEDIAWIKI_PAGE

$mediawiki_page .= join "\n", map {"* [[Eurodeputes_$_|$committee_meps{$_}->{NAME}]]"} sort keys %committee_meps;
$mediawiki_page .= "\n\n----\n\n=== Par pays ===\n\n";
$mediawiki_page .= join "\n", map {"* [[Eurodeputes_".$_."_par_commission|Eurodéputés ".($committee_country_meps{$_}->{NAME} =~ /^[AEIOUY]/o ? "d'" : /^(?:DK|GB|LU|PT)$/o ? "du " : $_ eq 'NL' ? "des " : "de ")."$committee_country_meps{$_}->{NAME} par commission parlementaire]]"} sort keys %committee_country_meps;
$mediawiki_page .= "\n\n----\n\nRetour à la page d'accueil de [[Memoire_politique|Mémoire politique]]\n";
$mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

# Output mediawiki page
my $mediawiki_location = "$output_dir/Eurodeputes_par_commission.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki pages for each committee
foreach my $committee_code (sort keys %committee_meps) {
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
  __NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_$committee_code|English]]''' - '''[[Eurodeputes_$committee_code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $committee_meps{$committee_code}->{NAME}, coordonnées des eurodéputés==

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Eurodéputé(e) !! Pays !! Groupe !! Contact !! Bureaux\n";
  $mediawiki_page .= "|-\n";
  foreach my $deputy (@{$committee_meps{$committee_code}->{MEPS}}) {
    (my $grp_wiki = $deputy->{GRP}) =~ s![/-].+$!!o;
    $mediawiki_page .= "| $deputy->{NAME} || $deputy->{COUNTRY} || {{#icon:$grp_wiki.png|$deputy->{GRP}|Eurodeputes_$grp_wiki}} [[Eurodeputes_$grp_wiki|$deputy->{GRP}]] || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL} || Bruxelles : $deputy->{BXL}<br />Strasbourg : $deputy->{STR}\n";
    $mediawiki_page .= "|-\n";
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n''Les eurodéputé(e)s dont le nom est en italique sont membres suppléant(e)s de la $committee_meps{$committee_code}->{NAME}.''\n";
  $mediawiki_page .= "\n-----\n\n[[Eurodeputes_par_commission|Autres commissions parlementaires]]\n";
  $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

  # Output mediawiki page
  my $mediawiki_location = "$output_dir/Eurodeputes_$committee_code.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

# Build mediawiki page for navigation by country
foreach my $country_code (sort keys %committee_country_meps) {
  my $country = $committee_country_meps{$country_code}->{NAME};
  my $preposition = $country =~ /^[AEIOUY]/o ? "d'" : $country_code =~ /^(?:DK|GB|LU|PT)$/o ? "du " : $country_code eq 'NL' ? "des " : "de ";
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
  __NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_${country_code}_by_committee|English]]''' - '''[[Eurodeputes_${country_code}_par_commission|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : coordonnées des eurodéputés $preposition$country {{#icon:$country_code.png|$country}} par commission parlementaire==

MEDIAWIKI_PAGE

  $mediawiki_page .= join "\n", map {"* [[Eurodeputes_${country_code}_$_|$committee_country_meps{$country_code}->{COMS}->{$_}->{NAME}]]"} sort keys %{$committee_country_meps{$country_code}->{COMS}};
  $mediawiki_page .= "\n\n----\n\n[[Eurodeputes_par_commission|Eurodéputés d'autres pays par commission parlementaire]]\n";
  $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

  # Output mediawiki page
  my $mediawiki_location = "$output_dir/Eurodeputes_${country_code}_par_commission.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";

  # Build mediawiki pages for each committee by country
  foreach my $committee_code (sort keys %{$committee_country_meps{$country_code}->{COMS}}) {
    my $mediawiki_page = << "MEDIAWIKI_PAGE";
  __NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_${country_code}_$committee_code|English]]''' - '''[[Eurodeputes_${country_code}_$committee_code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $committee_country_meps{$country_code}->{COMS}->{$committee_code}->{NAME}, coordonnées des eurodéputés $preposition$country {{#icon:$country_code.png|$country}}==

MEDIAWIKI_PAGE

    $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
    $mediawiki_page .= "! Eurodéputé(e) !! Groupe !! Contact !! Bureaux\n";
    $mediawiki_page .= "|-\n";
    foreach my $deputy (@{$committee_country_meps{$country_code}->{COMS}->{$committee_code}->{MEPS}}) {
      (my $grp_wiki = $deputy->{GRP}) =~ s![/-].+$!!o;
      $mediawiki_page .= "| $deputy->{NAME} || {{#icon:$grp_wiki.png|$deputy->{GRP}||Eurodeputes_$grp_wiki}} [[Eurodeputes_$grp_wiki|$deputy->{GRP}]] || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL} || Bruxelles : $deputy->{BXL}<br />Strasbourg : $deputy->{STR}\n";
      $mediawiki_page .= "|-\n";
    }
    $mediawiki_page =~ s/\|-\n$/|}\n/so;
    $mediawiki_page .= "\n''Les eurodéputé(e)s dont le nom est en italique sont membres suppléant(e)s de la $committee_meps{$committee_code}->{NAME}.''\n";
    $mediawiki_page .= "\n-----\n\n[[Eurodeputes_${country_code}_par_commission|Autres commissions parlementaires d'eurodéputés $preposition$country]]\n";
    $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

    # Output mediawiki page
    my $mediawiki_location = "$output_dir/Eurodeputes_${country_code}_$committee_code.mediawiki";
    open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
    print MEDIAWIKI $mediawiki_page;
    close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
    warn "$mediawiki_location done\n";
  }
}
