#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'meps.dbxml';
my $output_dir = "$root_dir/mediawiki/meps_offices";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: meps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/meps_names
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

my %bxl_meps;
my %str_meps;

my %buildings_order = (
  'ASP' => 1,
  'PHS' => 2,
  'ATR' => 3,
  'WIB' => 4,
  'LOW' => 5,
  'WIC' => 6,
  'SDM' => 7,
  'SDF' => 8,
);

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query = <<"QUERY";
for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician
return
  <deputy>
    <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "/fr|", \$deputy/infos/name/full/string(), "]]") }</name>
    <country>{ concat("{{#icon:", \$deputy/infos/constituency/country/code/string(), ".png|", \$deputy/infos/constituency/country/name/string(), "||Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "}} [[Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "|", \$deputy/infos/constituency/country/name/string(), "]]") }</country>
    <grp>{ \$deputy/infos/group/abbreviation/string() }</grp>
    <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "@", "(à)"), "]"), "<br />") }</email>
    <bxl>{ \$deputy/contact/address[label="Parlement européen"][city="Bruxelles"] }</bxl>
    <str>{ \$deputy/contact/address[label="Parlement européen"][city="Strasbourg Cedex"] }</str>
  </deputy>
QUERY
    #  Perform the query
    eval {
        my $results = $theMgr->query($query);
        my $value;
        while( $results->next($value) ) {
          my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
          my $mp = $xml->root;
          my %mp_data;
          $mp_data{NAME} = $mp->first_child_text('name');
          $mp_data{COUNTRY} = $mp->first_child_text('country');
          $mp_data{GRP} = $mp->first_child_text('grp');
          $mp_data{EMAIL} = $mp->first_child_text('email');

          my $bxl_addr = $mp->first_child('bxl')->first_child('address');
          my $bxl_building = $bxl_addr->first_child('building')->first_child_text('name');
          my $bxl_building_code = $bxl_addr->first_child('building')->first_child_text('abbreviation');
          my $bxl_office = $bxl_addr->first_child_text('office');
          my ($bxl_floor, $bxl_block);
          if ($bxl_office ne 'En cours de déménagement') {
            ($bxl_floor, $bxl_block, $mp_data{BXL_OFFICE}) =
              ($bxl_office =~ /(\d\d)([A-Z])0?(\d\d\d(?:[\/\-]\d\d\d)?)/o);
          }
          else {
            $bxl_building_code = 'SDF';
            $bxl_floor = $bxl_block = $mp_data{BXL_OFFICE} = '-';
          }
          $mp_data{BXL_TEL} = $bxl_addr->first_child_text('phone') || '-';
          if ($mp_data{BXL_TEL} ne '-') {
            (my $tel_url = $mp_data{BXL_TEL}) =~ s/ //go;
            $mp_data{BXL_TEL} = "[callto://$tel_url $mp_data{BXL_TEL}]";
          }
          $mp_data{BXL_FAX} = $bxl_addr->first_child_text('fax') || '-';
          $bxl_meps{$bxl_building_code}->{NAME} = $bxl_building;
          push @{$bxl_meps{$bxl_building_code}->{FLOORS}->{$bxl_floor}->{$bxl_block}}, \%mp_data;

          my $str_addr = $mp->first_child('str')->first_child('address');
          my $str_building = $str_addr->first_child('building')->first_child_text('name');
          my $str_building_code = $str_addr->first_child('building')->first_child_text('abbreviation');
          my $str_office = $str_addr->first_child_text('office');
          my $str_floor;
          if ($str_office ne 'En cours de déménagement') {
            ($str_floor, $mp_data{STR_OFFICE}) =
              ($str_office =~ /[TM](\d\d)(\d\d\d[ABC]?)/o);
          }
          else {
            $str_building_code = 'SDF';
            $str_floor = $mp_data{STR_OFFICE} = '-';
          }
          $mp_data{STR_TEL} = $bxl_addr->first_child_text('phone') || '-';
          if ($mp_data{STR_TEL} ne '-') {
            (my $tel_url = $mp_data{STR_TEL}) =~ s/ //go;
            $mp_data{STR_TEL} = "[callto://$tel_url $mp_data{BXL_TEL}]";
          }
          $mp_data{STR_FAX} = $bxl_addr->first_child_text('fax') || '-';
          $str_meps{$str_building_code}->{NAME} = $str_building;
          push @{$str_meps{$str_building_code}->{FLOORS}->{$str_floor}}, \%mp_data;
        }
	  };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

# Build mediawiki page for navigation
my $top_mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_by_office|English]]''' - '''[[Eurodeputes_par_bureau|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : coordonnées des eurodéputés par bureau==

MEDIAWIKI_PAGE

$top_mediawiki_page .= "===Bruxelles===\n";
foreach my $bxl_building_code (sort {$buildings_order{$a} <=> $buildings_order{$b}} keys %bxl_meps) {
  if ($bxl_building_code ne 'SDF') {
    $top_mediawiki_page .= "\n====Bâtiment $bxl_meps{$bxl_building_code}->{NAME}====\n\n";
    foreach my $bxl_floor_str (sort keys %{$bxl_meps{$bxl_building_code}->{FLOORS}}) {
      my $bxl_floor = $bxl_floor_str + 0;
      my $suffix = ($bxl_floor == 1 ? 'er' : 'e');
      $top_mediawiki_page .= "* [[Eurodeputes_${bxl_building_code}_$bxl_floor_str|$bxl_floor<sup>$suffix</sup> étage]]\n";
      
      # Build mediawiki page for each Brussels/building/floor
      my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_${bxl_building_code}_$bxl_floor_str|English]]''' - '''[[Eurodeputes_${bxl_building_code}_$bxl_floor_str|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : Bruxelles, Bâtiment $bxl_meps{$bxl_building_code}->{NAME}, $bxl_floor<sup>$suffix</sup> étage, coordonnées des eurodéputés==

MEDIAWIKI_PAGE

      # One table per block
      foreach my $bxl_block (sort keys %{$bxl_meps{$bxl_building_code}->{FLOORS}->{$bxl_floor_str}}) {
        $mediawiki_page .= "===Bloc $bxl_block===\n\n";
        $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
        $mediawiki_page .= "! Bureau !! Eurodéputé(e) !! Pays !! Groupe !! Téléphone(s) !! Fax !! Courriel\n";
        $mediawiki_page .= "|-\n";
        foreach my $deputy (sort {$a->{BXL_OFFICE} cmp $b->{BXL_OFFICE}} @{$bxl_meps{$bxl_building_code}->{FLOORS}->{$bxl_floor_str}->{$bxl_block}}) {
          (my $grp_wiki = $deputy->{GRP}) =~ s![/-].+$!!o;
          $mediawiki_page .= "| $deputy->{BXL_OFFICE} || $deputy->{NAME} || $deputy->{COUNTRY} || {{#icon:$grp_wiki.png|$deputy->{GRP}||Eurodeputes_$grp_wiki}} [[Eurodeputes_$grp_wiki|$deputy->{GRP}]] || $deputy->{BXL_TEL} || $deputy->{BXL_FAX} || $deputy->{EMAIL}\n";
          $mediawiki_page .= "|-\n";
        }
        #{
        $mediawiki_page =~ s/\|-\n$/|}\n\n/so;
      }
      $mediawiki_page .= "\n-----\n\n[[Eurodeputes_par_bureau|Autres bureaux]]\n";
      $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";
      
      # Output mediawiki page
      my $mediawiki_location = "$output_dir/Eurodeputes_${bxl_building_code}_$bxl_floor_str.mediawiki";
      open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
      print MEDIAWIKI $mediawiki_page;
      close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
      warn "$mediawiki_location done\n";
    }
  }
  else {
    $top_mediawiki_page .= "\n====$bxl_meps{$bxl_building_code}->{NAME}====\n\n";
    $top_mediawiki_page .= "* [[Eurodeputes_BXL_$bxl_building_code|Aucun bureau actuellement affecté]]\n";

    # Build mediawiki page for homeless MEPs in Brussels
    my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_BXL_$bxl_building_code|English]]''' - '''[[Eurodeputes_BXL_$bxl_building_code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : Bruxelles, coordonnées des eurodéputés en cours de déménagement==

MEDIAWIKI_PAGE

    $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
    $mediawiki_page .= "! Eurodéputé(e) !! Pays !! Groupe !! Téléphone(s) !! Fax !! Courriel\n";
    $mediawiki_page .= "|-\n";
    foreach my $deputy (sort @{$bxl_meps{$bxl_building_code}->{FLOORS}->{'-'}->{'-'}}) {
      (my $grp_wiki = $deputy->{GRP}) =~ s![/-].+$!!o;
      $mediawiki_page .= "| $deputy->{NAME} || $deputy->{COUNTRY} || {{#icon:$grp_wiki.png|$deputy->{GRP}||Eurodeputes_$grp_wiki}} [[Eurodeputes_$grp_wiki|$deputy->{GRP}]] || $deputy->{BXL_TEL} || $deputy->{BXL_FAX} || $deputy->{EMAIL}\n";
      $mediawiki_page .= "|-\n";
    }
    #{
    $mediawiki_page =~ s/\|-\n$/|}\n\n/so;
    $mediawiki_page .= "\n-----\n\n[[Eurodeputes_par_bureau|Liste des bureaux]]\n";
    $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

    # Output mediawiki page
    my $mediawiki_location = "$output_dir/Eurodeputes_BXL_$bxl_building_code.mediawiki";
    open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
    print MEDIAWIKI $mediawiki_page;
    close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
    warn "$mediawiki_location done\n";
  }
}

$top_mediawiki_page .= "\n===Strasbourg===\n";
foreach my $str_building_code (sort {$buildings_order{$a} <=> $buildings_order{$b}} keys %str_meps) {
  if ($str_building_code ne 'SDF') {
    $top_mediawiki_page .= "\n====Bâtiment $str_meps{$str_building_code}->{NAME}====\n\n";
    foreach my $str_floor_str (sort keys %{$str_meps{$str_building_code}->{FLOORS}}) {
      my $str_floor = $str_floor_str + 0;
      my $suffix = ($str_floor == 1 ? 'er' : 'e');
      $top_mediawiki_page .= "* [[Eurodeputes_${str_building_code}_$str_floor_str|$str_floor<sup>$suffix</sup> étage]]\n";
      
      # Build mediawiki page for each Strasbourg/building/floor
      my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_${str_building_code}_$str_floor_str|English]]''' - '''[[Eurodeputes_${str_building_code}_$str_floor_str|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : Strasbourg, Bâtiment $str_meps{$str_building_code}->{NAME}, $str_floor<sup>$suffix</sup> étage, coordonnées des eurodéputés==

MEDIAWIKI_PAGE

      $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
      $mediawiki_page .= "! Bureau !! Eurodéputé(e) !! Pays !! Groupe !! Téléphone(s) !! Fax !! Courriel\n";
      $mediawiki_page .= "|-\n";
      foreach my $deputy (sort {$a->{STR_OFFICE} cmp $b->{STR_OFFICE}} @{$str_meps{$str_building_code}->{FLOORS}->{$str_floor_str}}) {
        (my $grp_wiki = $deputy->{GRP}) =~ s![/-].+$!!o;
        $mediawiki_page .= "| $deputy->{STR_OFFICE} || $deputy->{NAME} || $deputy->{COUNTRY} || {{#icon:$grp_wiki.png|$deputy->{GRP}||Eurodeputes_$grp_wiki}} [[Eurodeputes_$grp_wiki|$deputy->{GRP}]] || $deputy->{STR_TEL} || $deputy->{STR_FAX} || $deputy->{EMAIL}\n";
        $mediawiki_page .= "|-\n";
      }
      #{
      $mediawiki_page =~ s/\|-\n$/|}\n\n/so;
      $mediawiki_page .= "\n-----\n\n[[Eurodeputes_par_bureau|Autres bureaux]]\n";
      $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";
      
      # Output mediawiki page
      my $mediawiki_location = "$output_dir/Eurodeputes_${str_building_code}_$str_floor_str.mediawiki";
      open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
      print MEDIAWIKI $mediawiki_page;
      close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
      warn "$mediawiki_location done\n";
    }
  }
  else {
    $top_mediawiki_page .= "\n====$str_meps{$str_building_code}->{NAME}====\n\n";
    $top_mediawiki_page .= "* [[Eurodeputes_STR_$str_building_code|Aucun bureau actuellement affecté]]\n";

    # Build mediawiki page for homeless MEPs in Strasbourg
    my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_STR_$str_building_code|English]]''' - '''[[Eurodeputes_STR_$str_building_code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : Strasbourg, coordonnées des eurodéputés en cours de déménagement==

MEDIAWIKI_PAGE
    $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
    $mediawiki_page .= "! Eurodéputé(e) !! Pays !! Groupe !! Téléphone(s) !! Fax !! Courriel\n";
    $mediawiki_page .= "|-\n";
    foreach my $deputy (sort @{$str_meps{$str_building_code}->{FLOORS}->{'-'}}) {
      (my $grp_wiki = $deputy->{GRP}) =~ s![/-].+$!!o;
      $mediawiki_page .= "| $deputy->{NAME} || $deputy->{COUNTRY} || {{#icon:$grp_wiki.png|$deputy->{GRP}||Eurodeputes_$grp_wiki}} [[Eurodeputes_$grp_wiki|$deputy->{GRP}]] || $deputy->{STR_TEL} || $deputy->{STR_FAX} || $deputy->{EMAIL}\n";
      $mediawiki_page .= "|-\n";
    }
    #{
    $mediawiki_page =~ s/\|-\n$/|}\n\n/so;
    $mediawiki_page .= "\n-----\n\n[[Eurodeputes_par_bureau|Liste des bureaux]]\n";
    $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

    # Output mediawiki page
    my $mediawiki_location = "$output_dir/Eurodeputes_STR_$str_building_code.mediawiki";
    open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
    print MEDIAWIKI $mediawiki_page;
    close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
    warn "$mediawiki_location done\n";
  }
}

$top_mediawiki_page .= "\n----\n\nRetour à la page d'accueil de [[Memoire_politique|Mémoire politique]]\n";
$top_mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

# Output mediawiki page for navigation
my $top_mediawiki_location = "$output_dir/Eurodeputes_par_bureau.mediawiki";
open TOP_MEDIAWIKI, ">$top_mediawiki_location" or die "Cannot write in file $top_mediawiki_location: $!\n";
print TOP_MEDIAWIKI $top_mediawiki_page;
close TOP_MEDIAWIKI or die "Error when closing $top_mediawiki_location: $!\n";
warn "$top_mediawiki_location done\n";
