#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;
use Tie::IxHash;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'mps.dbxml';
my $output_dir = "$root_dir/mediawiki/mps";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir] <MemberOfParliamentWikiName>
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically mps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: mps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/mps
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

my $mp_wiki = shift or die "You should provide the wiki name of the Member of the Parliament, see $0 -h\n";

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv,
          Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml");

# Get the document
eval {
  my $theDocument = $container->getDocument($mp_wiki);
  my $docString = $theDocument->getContent();

  # Build xml tree
  my $xml = XML::Twig->new(keep_encoding => 1)->parse($docString);

  # Extract infos
  my $infos = $xml->root->first_child('infos');
  my $name = $infos->first_child('name');
  my $gender = $name->first_child_text('gender');
  my $role = 'député'.(($gender eq 'M.')?'':'e');
  my $first_name = $name->first_child_text('first');
  my $last_name = $name->first_child_text('last');
  my $wiki_name = $name->first_child_text('wiki');
  my $pic = $infos->first_child_text('picture');
  my $birth = $infos->first_child('birth');
  my $birth_date = $birth->first_child('date');
  my $birth_place = $birth->first_child('place');
  my $birth_str = sprintf "''%s le'' %02d %s %04d ''à'' %s (%s)", $birth->first_child_text('gender'), $birth_date->first_child_text('day'), $birth_date->first_child_text('month'), $birth_date->first_child_text('year'), $birth_place->first_child_text('city'), $birth_place->first_child_text('department');
  my $constituency = $infos->first_child('constituency');
  my $constituency_dep = sprintf "%s (%s)", $constituency->first_child('department')->first_child_text('name'), $constituency->first_child('department')->first_child_text('number');
  my $constituency_nb = $constituency->first_child_text('number');
  my $constituency_cantons = join ', ', $constituency->first_child('cantons')->children_text('name');
  my $constituency_wiki = $constituency->first_child('department')->first_child_text('number');
  my $grp = $infos->first_child('group');
  my $grp_str = $grp->first_child_text('name');
  my $grp_ap = $grp->att('status') if $grp->att('status');
  $grp_ap .= 'e' if $grp_ap and $gender ne 'M.';
  $grp_str = "$grp_ap $grp_str" if $grp_ap;
  $grp_str .= ' ('.$grp->att('role').')' if $grp->att('role');
  my $party = $grp->first_child_text('abbreviation');
  my $grp_wiki;
  if ($grp->first_child_text('name') eq "Députés n'appartenant à aucun groupe") {
    $grp_wiki = 'NI'
  }
  else {
    ($grp_wiki = $party) =~ s/^Ap\. //o;
    $grp_wiki = uc($grp_wiki);
  }
  my $prof = $infos->first_child_text('profession');

  my $contact = $xml->root->first_child('contact');
  my $address = "* " . join("\n* ", map({($_->first_child('label')?"''".$_->first_child_text('label')."'' ":'').($_->first_child('street')?$_->first_child_text('street').',':'').($_->first_child('postcode')?' '.$_->first_child_text('postcode'):'').($_->first_child('city')?' '.$_->first_child_text('city'):'').($_->first_child('phone')?(" <br />''Tél.'' : ".join('/', (map {(my $phone_url = $_) =~ s/ //g;$phone_url =~ s/^0/+33/;"[callto://$phone_url $_]"} $_->children_text('phone')))):'').($_->first_child('fax')?(' - Fax : '.join(' - Fax : ', $_->children_text('fax'))):'')} $contact->children('address')))."\n";

  # Hack for telephone/fax without address
  $address =~ s/^\*  - /* /om;

  (my $icon_phone) = grep /^01 40 6/, map {$_->text} $contact->descendants('phone');
  if ($icon_phone) {
    (my $url_phone = $icon_phone) =~ s/ //g;
    $url_phone =~ s/^0/+33/;
    $icon_phone = "{{#icon:Cliquez_pour_appeler.png|$icon_phone||callto://$url_phone}} ";
  }
  else {
    $icon_phone = '';
  }

  my @mails = map {
    (my $mail_str = $_->text) =~ s/@/(à)/o;
    [$_->text(), $mail_str];
  } $contact->children('email');
  my @webs = map {
    (my $web_str = $_->text) =~ s#^http://##o;
    my $type=$_->att('type')?ucfirst($_->att('type')):'Site web';
    [$_->text(), $web_str, $type];
  } $contact->children('web');

  # Group functions by type
  my $functions = $xml->root->first_child('functions');
  my %function;
  foreach my $func ($functions->children('function')) {
    my $func_val;
    my $type = $func->first_child_text('type');
    my $label = $func->first_child_text('label');
    my $role = $func->first_child_text('role');
    my $mission = $func->first_child_text('mission');
    if ($type eq 'Commission') {
      my $com_wiki = wikify_com($label);
      if ($com_wiki) {
        $label = "[[Deputes_$com_wiki|$label]]";
      }
    }
    if ($mission) {
      $func_val = "$role au nom de la $label ($mission)";
    }
    else {
      $func_val = ($label?$label:'').($role?" ($role)":'');
    }
    if (exists $function{$type}) {
      $function{$type} .= ", $func_val";
    }
    else {
      if ($type eq 'Mandat') {
        $function{$func_val} = '';
      }
      else {
        $function{$type} = $func_val;
      } 
    }
  }

  # Group mandates by type and by institution or role
  my $mandates = $xml->root->first_child('mandates');
  my %mandate;
  foreach my $mandate ($mandates->children('mandate')) {
    my $type = $mandate->first_child_text('type');
    if ($mandate->att('current') eq 'true') {
      $type .= ' en cours' if $type ne 'Organismes extra-parlementaires';
      # Hack for "Mandats et fonctions à l'Assemblée nationale"
      $type =~ s/Mandats et fonctions à l'Assemblée nationale en cours/Mandats et fonctions en cours à l'Assemblée nationale/o;
    }
    else {
      $type =~ s/^Mandats/Anciens mandats/o;
      $type =~ s/^Fonctions/Anciennes fonctions/o;
    }
    my $institution_or_role = 'null';
    my $mandate_str = '';
    if ($mandate->first_child('institution')) {
      $institution_or_role = $mandate->first_child_text('institution');
      if ($mandate->first_child('election_date')) {
        $mandate_str .= sprintf("Élections du %02d/%02d/%04d - ", $mandate->first_child('election_date')->first_child_text('day'), $mandate->first_child('election_date')->first_child_text('month'), $mandate->first_child('election_date')->first_child_text('year'));
      }
      if ($mandate->first_child('begin_term')) {
        $mandate_str .= sprintf("Mandat du %02d/%02d/%04d ", $mandate->first_child('begin_term')->first_child_text('day'), $mandate->first_child('begin_term')->first_child_text('month'), $mandate->first_child('begin_term')->first_child_text('year'));
        $mandate_str .= sprintf("(%s) ", $mandate->first_child('begin_term')->first_child_text('reason')) if $mandate->first_child('begin_term')->first_child('reason');
      }
      if ($mandate->first_child('end_term')) {
        $mandate_str .= sprintf("au %02d/%02d/%04d", $mandate->first_child('end_term')->first_child_text('day'), $mandate->first_child('end_term')->first_child_text('month'), $mandate->first_child('end_term')->first_child_text('year'));
        $mandate_str .= sprintf(" (%s)", $mandate->first_child('end_term')->first_child_text('reason')) if $mandate->first_child('end_term')->first_child('reason');
      }
      $mandate_str .= sprintf(" : %s", $mandate->first_child_text('role')) if $mandate->first_child('role');
    }
    elsif ($mandate->first_child('role')) {
      $institution_or_role = $mandate->first_child_text('role');
      if ($mandate->first_child('begin_term')) {
        $mandate_str .= sprintf("Mandat du %02d/%02d/%04d ", $mandate->first_child('begin_term')->first_child_text('day'), $mandate->first_child('begin_term')->first_child_text('month'), $mandate->first_child('begin_term')->first_child_text('year'));
        $mandate_str .= sprintf("(%s) ", $mandate->first_child('begin_term')->first_child_text('reason')) if $mandate->first_child('begin_term')->first_child('reason');
      }
      if ($mandate->first_child('end_term')) {
        $mandate_str .= sprintf("au %02d/%02d/%04d", $mandate->first_child('end_term')->first_child_text('day'), $mandate->first_child('end_term')->first_child_text('month'), $mandate->first_child('end_term')->first_child_text('year'));
        $mandate_str .= sprintf(" (%s)", $mandate->first_child('end_term')->first_child_text('reason')) if $mandate->first_child('end_term')->first_child('reason');
      }
    }
    else {
      if ($mandate->first_child('election_date')) {
        $mandate_str .= sprintf("Élections du %02d/%02d/%04d - ", $mandate->first_child('election_date')->first_child_text('day'), $mandate->first_child('election_date')->first_child_text('month'), $mandate->first_child('election_date')->first_child_text('year'));
      }
      if ($mandate->first_child('begin_term')) {
        $mandate_str .= sprintf("Mandat du %02d/%02d/%04d ", $mandate->first_child('begin_term')->first_child_text('day'), $mandate->first_child('begin_term')->first_child_text('month'), $mandate->first_child('begin_term')->first_child_text('year'));
        $mandate_str .= sprintf("(%s) ", $mandate->first_child('begin_term')->first_child_text('reason')) if $mandate->first_child('begin_term')->first_child('reason');
      }
      if ($mandate->first_child('end_term')) {
        $mandate_str .= sprintf("au %02d/%02d/%04d", $mandate->first_child('end_term')->first_child_text('day'), $mandate->first_child('end_term')->first_child_text('month'), $mandate->first_child('end_term')->first_child_text('year'));
        $mandate_str .= sprintf(" (%s)", $mandate->first_child('end_term')->first_child_text('reason')) if $mandate->first_child('end_term')->first_child('reason');
      }
      $mandate_str .= sprintf(" (%s)", $mandate->first_child_text('role')) if $mandate->first_child('role');
    }

    if (exists $mandate{$type}) {
      if (exists($mandate{$type}{$institution_or_role})) {
        push @{$mandate{$type}{$institution_or_role}}, $mandate_str;
      }
      else {
        $mandate{$type}{$institution_or_role} = [$mandate_str];
      }
    }
    else {
      # Second level is an ordered hash to keep chronology
      my %ordered_hash;
      tie %ordered_hash, "Tie::IxHash";
      %ordered_hash = ($institution_or_role => [$mandate_str]);
      $mandate{$type} = \%ordered_hash;
    }
  }

  my @positions=();
  if ($xml->root->first_child('cv')) {
    @positions = map {$_->text} $xml->root->first_child('cv')->children('position');
  }

  my $activities = $xml->root->first_child('activities');
  (my $questions_url = $activities->first_child_text('questions')) =~ s/ /%20/go;
  (my $propositions_url = $activities->first_child_text('propositions')) =~ s/ /%20/go;
  (my $reports_url = $activities->first_child_text('reports')) =~ s/ /%20/go;
  (my $speeches_url = $activities->first_child_text('speeches')) =~ s/ /%20/go;
  (my $commissions_url = $activities->first_child_text('commissions')) =~ s/ /%20/go;
  (my $debates_url = $activities->first_child_text('debates')) =~ s/ /%20/go;

  my @opinions=();
  if ($xml->root->first_child('opinions')) {
    @opinions = map {"* ".$_->first_child_text('date')." [".$_->first_child_text('url')." ".$_->first_child_text('title')."]\n\n".$_->first_child_text('content')."\n"} $xml->root->first_child('opinions')->children('opinion');
  }

  # Build mediawiki page
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
==[[Memoire_politique|Mémoire politique]] : $first_name $last_name, $role==

[[Image:$pic|right|$first_name $last_name]]

===Informations générales===

* $birth_str
* ''Circonscription d'élection'' : [[Deputes_$constituency_wiki|$constituency_dep]], $constituency_nb circonscription<br/>''Cantons de'' $constituency_cantons
* ''Groupe politique'' : [[Deputes_$grp_wiki|$grp_str]]<br/>''Parti'' : $party
MEDIAWIKI_PAGE
  
  $mediawiki_page .= "* ''Profession'' : $prof\n" if $prof;

  $mediawiki_page .= << "MEDIAWIKI_PAGE";
{{Robelbox|theme=13|title=Contact|icon=Calligraphy.png|iconwidth=40px|width=60%}}
<div style="{{Robelbox/pad}}">
<div style="float:right">$icon_phone</div>
$address
MEDIAWIKI_PAGE
  
  foreach my $mail (@mails) {
    $mediawiki_page .= "* ''Courriel'' : [mailto:$mail->[0] $mail->[1]]\n";
  }
  foreach my $web (@webs) {
    $mediawiki_page .= "* ''$web->[2]'' : [$web->[0] $web->[1]]\n";
  }

  $mediawiki_page .= << "MEDIAWIKI_PAGE";
</div>
{{Robelbox/close}}

MEDIAWIKI_PAGE

  $mediawiki_page .= "\n====Fonctions à l'Assemblée nationale====\n\n";
  foreach my $function_type (sort keys %function) {
    if ($function{$function_type}) {
      $mediawiki_page .= "* ''$function_type'' : $function{$function_type}\n";
    }
    else {
      $mediawiki_page .= "* $function_type\n";
    }
  }

  $mediawiki_page .= "\n====Mandats====\n\n";
  foreach my $mandate_type (sort sort_mandate_type keys %mandate) {
    $mediawiki_page .= "* ''$mandate_type''\n";
    foreach my $mandate_second_level (keys %{$mandate{$mandate_type}}) {
      if ($mandate_second_level eq 'null') {
        $mediawiki_page .= "** ".join("\n** ", @{$mandate{$mandate_type}{$mandate_second_level}})."\n";
      }
      else {
        $mediawiki_page .= "** $mandate_second_level\n";
        if ($mandate{$mandate_type}{$mandate_second_level}[0]) {
          $mediawiki_page .= "*** ".join("\n*** ", @{$mandate{$mandate_type}{$mandate_second_level}})."\n";
        }
      }
    }
  }

  if (scalar @positions) {
    $mediawiki_page .= "\n====Curriculum Vitae====\n\n";
    foreach my $pos (@positions) {
      $mediawiki_page .= "* $pos\n";
    }
  }

  $mediawiki_page .= <<"MEDIAWIKI";

----

===Prises de positions===

====Sources d'informations====

* [$questions_url Questions posées à l'Assemblée nationale]
* [$propositions_url Propositions de loi et de résolution à l'Assemblée nationale]
* [$reports_url Rapports à l'Assemblée nationale]
* [$speeches_url Interventions en séance à l'Assemblée nationale]
* [$commissions_url Interventions en réunions de commission à l'Assemblée nationale]
MEDIAWIKI

$mediawiki_page .= "* [$debates_url Table nominative des débats à l'Assemblée nationale]\n" if $debates_url;

    $mediawiki_page .= <<"MEDIAWIKI";
* [http://fr.wikipedia.org/wiki/${first_name}_${last_name} Wikipédia]
* [http://www.google.fr/search?q=$first_name+$last_name Google]

====Positions====

''Merci d'enrichir cette partie en y rapportant les prises de positions de $first_name $last_name concernant les sujets traités par La Quadrature du Net (consultez la page [[Aide:Memoire_politique]] pour savoir comment faire).''

MEDIAWIKI

  foreach my $opinion (@opinions) {
    $opinion =~ s/&lt;/</go;
    $mediawiki_page .= "$opinion\n";
  }

  $mediawiki_page .= "[[Category:Députés]] [[Category:Mémoire_Politique]]\n";

  # Output mediawiki page
  my $mediawiki_location = "$output_dir/$wiki_name.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";

};

if (my $e = catch std::exception) {
  warn "getDocument failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "getDocument failed\n";
  warn $@;
  exit( -1 );
}

sub sort_mandate_type {
  my $ordered_types = "
    Mandats et fonctions en cours à l'Assemblée nationale
    Anciens mandats et fonctions à l'Assemblée nationale
    Organismes extra-parlementaires
    Anciens mandats de sénateur
    Anciens mandats nationaux ou fonctions ministérielles
    Mandats locaux en cours
    Mandats intercommunaux en cours
    Anciens mandats locaux
    Anciens mandats intercommunaux
    Fonctions dans les instances internationales ou judiciaires en cours
    Anciennes fonctions dans les instances internationales ou judiciaires
    Anciens mandats européens
  ";
  return index($ordered_types, $a) <=> index($ordered_types, $b);
}

sub wikify_com {
  my $str = shift;
  my %coms = (
    'Commission des affaires culturelles, familiales et sociales' => 'CAFC',
    'Commission des affaires économiques, de l\'environnement et du territoire' => 'CPRO',
    'Commission des affaires étrangères' => 'CAFE',
    'Commission de la défense nationale et des forces armées' => 'CDEF',
    'Commission des finances' => 'CFIAB',
    'Commission des lois' => 'CLOI',
  );
  return $coms{$str} if (exists $coms{$str});
  return undef;
}
