#!/usr/bin/perl

# Copyright 2007 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use encoding 'utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $directory = "$root_dir/misc/annuairedesdeputes.txt";
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'mps.dbxml';

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'directory|d=s'  => \$directory,
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-d <directory_path>] [-p <path_to_db_env>]
Options can be:
  --directory, -d <directory_path>:  path to members of parliament's directory (text file)
                                     default: $root_dir/misc/annuairedesdeputes.txt
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically mps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(\d*)(?:\.dbxml)?$/s$1/;
my $legislature = $1;

# Parse directory text file
my %mps;
my $mp_name;
open DIRECTORY, $directory or die "Cannot open file $directory: $!\n";
while (<DIRECTORY>) {
  next if /^\s*$/o;
  if (/^Groupe Politique/) {
    if (/: (\d)(\d\d)(\d\d) \/ (\d)(\d\d)(\d\d)/) {
      $mps{wikify($mp_name)}->{TEL} = "$1 $2 $3";
      $mps{wikify($mp_name)}->{FAX} = "$4 $5 $6";
    }
  }
  else {
    chomp;
    $mp_name = $_;
  }
}

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv, Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml");
my $updateContext = $theMgr->createUpdateContext();

# Update each mp
foreach my $mp (sort keys %mps) {
  eval {
    # Get the document
    my $theDocument = $container->getDocument($mp);
    my $docString = $theDocument->getContent();
    my $xml = XML::Twig->new(output_encoding=>'UTF-8',pretty_print => 'indented')->parse($docString);
    my $contact = $xml->root->first_child('contact');
    my $address;
    foreach my $addr ($contact->children('address')) {
      next unless $addr->first_child_text('label') eq 'Assemblée nationale';
      $address = $addr;
    }
    unless ($address) {
      $address = $contact->insert_new_elt('last_child', 'address');
      $address->insert_new_elt('last_child', 'label', 'Assemblée nationale');
      $address->insert_new_elt('last_child', 'street', "126 rue de l'Université");
      $address->insert_new_elt('last_child', 'postcode', '75355');
      $address->insert_new_elt('last_child', 'city', 'Paris 07 SP');
    }
    if (my $phone = $address->first_child('phone')) {
      $phone->set_text("01 40 6$mps{$mp}->{TEL}");
    }
    else {
      $address->insert_new_elt('last_child','phone', "01 40 6$mps{$mp}->{TEL}");
    }
    if (my $fax = $address->first_child('fax')) {
      $fax->set_text("01 40 6$mps{$mp}->{FAX}");
    }
    else {
      $address->insert_new_elt('last_child','fax', "01 40 6$mps{$mp}->{FAX}");
    }

    # Update document into database
    $docString = $xml->sprint;
    $theDocument->setContent($docString);
    $container->updateDocument($theDocument, $updateContext);

    warn "$mp updated in container $theContainer\n";
  };
  if (my $e = catch std::exception) {
    warn "Exception while processing $mp:\n";
    warn $e->what() . "\n";
    exit(-1);
  }
  elsif ($@) {
    warn "Error while processing $mp: $@\n";
    exit(-1);
  }
}

sub wikify {
  my $str = shift;
  $str = desaccent($str);
  $str =~ s/-(.)/\u$1/go;
  # Dirty hacks for compounf names
  $str =~ s/CamilleDeroccaserra/CamilledeRoccaSerra/o;
  $str =~ s/CharlesDecourson/CharlesdeCourson/o;
  $str =~ s/FrancoiseDepanafieu/FrancoisedePanafieu/o;
  $str =~ s/JeanDionisdusejour/JeanDionisduSejour/o;
  $str =~ s/PierreMorelALhuissier/PierreMorelALHuissier/o;
  $str =~ s/RichardDellagnola/RichardDellAgnola/o;
  return $str;
}

sub desaccent {
  my $str = shift;
  my $mp_name = $str;
  $str =~ s/À/A/go;
  $str =~ s/Â/A/go;
  $str =~ s/Ä/A/go;
  $str =~ s/É/E/go;
  $str =~ s/È/E/go;
  $str =~ s/Ê/E/go;
  $str =~ s/Ë/E/go;
  $str =~ s/Í/I/go;
  $str =~ s/Î/I/go;
  $str =~ s/Ï/I/go;
  $str =~ s/Ó/O/go;
  $str =~ s/Ô/O/go;
  $str =~ s/Ö/O/go;
  $str =~ s/Ù/U/go;
  $str =~ s/Û/U/go;
  $str =~ s/Ü/U/go;
  $str =~ s/Ç/C/go;
  $str =~ s/à/a/go;
  $str =~ s/â/a/go;
  $str =~ s/ä/a/go;
  $str =~ s/é/e/go;
  $str =~ s/è/e/go;
  $str =~ s/ê/e/go;
  $str =~ s/ë/e/go;
  $str =~ s/í/i/go;
  $str =~ s/î/i/go;
  $str =~ s/ï/i/go;
  $str =~ s/ó/o/go;
  $str =~ s/ô/o/go;
  $str =~ s/ö/o/go;
  $str =~ s/ù/u/go;
  $str =~ s/û/u/go;
  $str =~ s/ü/u/go;
  $str =~ s/ç/c/go;
  $str =~ s/[ ']//go;
  return $str;
}
