#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'votes.dbxml';
my $theMepContainer = 'meps.dbxml';
my $output_dir = "$root_dir/mediawiki/votes";
my $detail = 0;

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s'    => \$path2DbEnv,
  'container|c=s'     => \$theContainer,
  'mep_container|m=s' => \$theMepContainer,
  'output|o=s'        => \$output_dir,
  'detail|d'          => \$detail,
  'help|h'            =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-hd] [-p <path_to_db_env>] [-c <container>] [-o <output_dir] <VoteWikiName>
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container for votes
                                     <container> is typically votes (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: votes.dbxml
  --mep_container, -m <container>:   DB XML container for MEPs
                                     <container> is typically meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: meps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/votes
  --detail, -d:                      output global result on a page and details on another page
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;
$theMepContainer =~ s/s?(?:\.dbxml)?$/s/;

my $detail_wiki = $detail ? '_details' : '';
my $detail_text = $detail ? ' détaillés' : '';

my $vote_wiki = shift or die "You should provide the wiki name of the vote, see $0 -h\n";

my %votes;
my %scores;
my @parts;
my $vote_label;

my @colours = (
  '#ff0000',
  '#ff1900',
  '#ff3300',
  '#ff4c00',
  '#ff6600',
  '#ff7f00',
  '#ff9900',
  '#ffb200',
  '#ffcc00',
  '#ffe500',
  '#ffff00',
  '#e5ff00',
  '#ccff00',
  '#b2ff00',
  '#99ff00',
  '#7fff00',
  '#66ff00',
  '#4cff00',
  '#33ff00',
  '#19ff00',
  '#00FF00',
);

# Dirty hack to have English translations of analyzed votes
my %translations = (
  Directive_brevets_logiciels_1re_lecture => 'Software_patents_directive_1st_reading',
  Resolution_EPLA                         => 'EPLA_resolution',
  Resolution_industries_culturelles       => 'Cultural_industries_resolution',
  Directives_Paquet_telecom_1re_lecture   => 'Telecoms_package_directives_1st_reading',
);
my %categories = (
  Directive_brevets_logiciels_1re_lecture => '[[Category:Brevets_Logiciels]] [[Category:Directives_fr]] [[Category:Mémoire_Politique]]',
  Resolution_EPLA                         => '[[Category:EPLA_fr]] [[Category:Résolutions]] [[Category:Mémoire_Politique]]',
  Resolution_industries_culturelles       => '[[Category:Industries_Culturelles]] [[Category:Résolutions]] [[Category:Mémoire_Politique]]',
  Directives_Paquet_telecom_1re_lecture   => '[[Category:Paquet_Telecom]] [[Category:Directives_fr]] [[Category:Mémoire_Politique]]',
);

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv,
          Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml");
my $mep_container = $theMgr->openContainer("$theMepContainer.dbxml");

# Get the document
eval {
  my $theDocument = $container->getDocument($vote_wiki);
  my $docString = $theDocument->getContent();

  # Build xml tree
  my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($docString);

  # Extract infos
  $vote_label = $xml->root->first_child_text('label');

  foreach my $vote_elt ($xml->root->children('vote')) {
    my $vote_subject = $vote_elt->first_child('subject');
    my $text = $vote_subject->first_child_text('text');
    if (exists $votes{TEXT}) {
      my $txt_regexp = $text;
      $txt_regexp =~ s/([\(\)\[\]\+\*\.\?])/\\$1/go;
      unless ($votes{TEXT} =~ /$txt_regexp/) {
        $votes{TEXT} .= " / $text";
      }
    }
    else {
      $votes{TEXT} = $text;
    }

    my $date_elt = $vote_elt->first_child('date');
    my $date = sprintf "%02d/%02d/%04d", $date_elt->first_child_text('day'), $date_elt->first_child_text('month'), $date_elt->first_child_text('year');
    if (exists $votes{DATE}) {
      unless ($votes{DATE} =~ /$date/) {
        $votes{DATE} .= " / $date";
      }
    }
    else {
      $votes{DATE} = $date;
    }

    my $part = $vote_subject->first_child_text('part');
    # Append text name in case of another text with the same part already exists
    $part .= "_$text" if $votes{VOTES}->{$part};

    $votes{VOTES}->{$part}->{DESCR} = $vote_subject->first_child_text('description');
    $votes{VOTES}->{$part}->{REC} = $vote_subject->first_child_text('recommendation');
    $votes{VOTES}->{$part}->{WEIGHT} = $vote_subject->first_child_text('weight');
    push @parts, $part;

    my $result_elt = $vote_elt->first_child('result');
    foreach my $mep ($result_elt->children('mep')) {
      my $name = $mep->first_child_text('name');
      unless (exists $scores{$name}) {
        if ($mep->first_child('dbxmlid')) {
          my $dbxmlid = $mep->first_child_text('dbxmlid');
          $scores{$name}->{DBXMLID} = $dbxmlid;

          # Get MEP document
          my $mepDocument = $mep_container->getDocument($dbxmlid);
          my $mepString = $mepDocument->getContent();
          my $mep_xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($mepString);

          my $mep_infos = $mep_xml->root->first_child('infos');
          $scores{$name}->{NAME}->{FIRST} = $mep_infos->first_child('name')->first_child_text('first');
          $scores{$name}->{NAME}->{LAST} = $mep_infos->first_child('name')->first_child_text('last');
          $scores{$name}->{COUNTRY}->{CODE} = $mep_infos->first_child('constituency')->first_child('country')->first_child_text('code');
          $scores{$name}->{COUNTRY}->{NAME} = $mep_infos->first_child('constituency')->first_child('country')->first_child_text('name');
          $scores{$name}->{GRP}->{CODE} = $mep_infos->first_child('group')->first_child_text('abbreviation');
          $scores{$name}->{GRP}->{NAME} = $mep_infos->first_child('group')->first_child_text('name');
          my $mep_functions = $mep_xml->root->first_child('functions');
          my%coms = map {$_->first_child_text('abbreviation') => $_->first_child_text('label')} grep {$_->first_child('abbreviation')} $mep_functions->children('function');
          $scores{$name}->{COMS} = \%coms;
        }
      }
      $scores{$name}->{VOTES}->{$part}->{VOTE} = $mep->first_child_text('choice');
      
      # Compute score for this vote
      # unless there is no recommendation
      next unless $votes{VOTES}->{$part}->{REC};
      if ($votes{VOTES}->{$part}->{REC} eq 'for') {
        if ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'for') {
          $scores{$name}->{VOTES}->{$part}->{SCORE} = 2;
        }
        elsif ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'against') {
          $scores{$name}->{VOTES}->{$part}->{SCORE} = -2;
        }
        else {
          $scores{$name}->{VOTES}->{$part}->{SCORE} = -1;
        }
      }
      elsif ($votes{VOTES}->{$part}->{REC} eq 'against') {
        if ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'for') {
          $scores{$name}->{VOTES}->{$part}->{SCORE} = -2;
        }
        elsif ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'against') {
          $scores{$name}->{VOTES}->{$part}->{SCORE} = 2;
        }
        else {
          $scores{$name}->{VOTES}->{$part}->{SCORE} = 1;
        }
      }
      # Weight
      $scores{$name}->{VOTES}->{$part}->{SCORE} *= $votes{VOTES}->{$part}->{WEIGHT};
    }
  }
};

if (my $e = catch std::exception) {
  warn "getDocument failed with exception\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "getDocument failed\n";
  warn $@;
  exit( -1 );
}

# Sump up weights for all votes
$votes{SUM_WEIGHTS} = 0;
foreach my $val (values %{$votes{VOTES}}) {
  $votes{SUM_WEIGHTS} += $val->{WEIGHT} if $val->{WEIGHT};
}

my $ep_score = 0;

foreach my $mep (keys %scores) {
  $scores{$mep}->{SUM_SCORES} = 0;
  # Count no vote as abstention
  # unless there is no recommendation
  foreach my $part (keys %{$votes{VOTES}}) {
    unless ($scores{$mep}->{VOTES}->{$part}) {
      next unless $votes{VOTES}->{$part}->{REC};
      if ($votes{VOTES}->{$part}->{REC} eq 'for') {
        $scores{$mep}->{VOTES}->{$part}->{SCORE} = -1 * $votes{VOTES}->{$part}->{WEIGHT};
      }
      elsif ($votes{VOTES}->{$part}->{REC} eq 'against') {
        $scores{$mep}->{VOTES}->{$part}->{SCORE} = $votes{VOTES}->{$part}->{WEIGHT};
      }
    }
  }

  # Sump up scores for all votes
  foreach my $val (values %{$scores{$mep}->{VOTES}}) {
    $scores{$mep}->{SUM_SCORES} += $val->{SCORE} if $val and $val->{SCORE};
  }

  # Compute final score from 0 to 100
  $scores{$mep}->{FINAL_SCORE} = sprintf("%.1f", 25 * ($scores{$mep}->{SUM_SCORES} / $votes{SUM_WEIGHTS}) + 50);
  $ep_score += 25 * ($scores{$mep}->{SUM_SCORES} / $votes{SUM_WEIGHTS}) + 50;
}
$ep_score = sprintf("%.1f", $ep_score / scalar(keys %scores));

#use Data::Dumper;
#print "VOTES=!".Dumper(\%votes)."!\nSCORES=!".Dumper(\%scores)."!\n";

# Build main mediawiki page
my $plural = ($votes{TEXT} =~ / \/ /o) ? 's' : '';
my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}|English]]''' - '''[[$vote_wiki|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label==

Vote sur le$plural texte$plural '''$votes{TEXT}''', le $votes{DATE}, résultats des votes par appel nominal sur les parties détaillées ci-dessous.

===Résultats===

'''Note moyenne du Parlement européen : $ep_score/100'''

* [[${vote_wiki}_par_note|Résultats classés par note finale]]
* [[${vote_wiki}_par_nom|Résultats classés par ordre alphabétique]]
* [[${vote_wiki}_par_pays|Résultats classés par pays]]
* [[${vote_wiki}_par_groupe|Résultats classés par groupe politique]]
* [[${vote_wiki}_par_commission|Résultats classés par commission parlementaire]]

===Votes par appel nominal===

MEDIAWIKI_PAGE

my $details = '';
foreach my $part (@parts) {
  my $rec =  $votes{VOTES}->{$part}->{REC} eq 'for' ? 'vote pour' :  $votes{VOTES}->{$part}->{REC} eq 'against' ? 'vote contre' : 'aucune';
  $details .= "* $part : $votes{VOTES}->{$part}->{DESCR} - recommandation : $rec".($votes{VOTES}->{$part}->{WEIGHT} ? ", coefficient : $votes{VOTES}->{$part}->{WEIGHT}" : '')."\n";
}
$mediawiki_page .= $details;

$mediawiki_page .= << "MEDIAWIKI_PAGE";

----

Retour à la page d'accueil de [[Memoire_politique|Mémoire politique]]
$categories{$vote_wiki}
MEDIAWIKI_PAGE

# Output main mediawiki page
my $mediawiki_location = "$output_dir/$vote_wiki.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki page by score
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}${detail_wiki}_by_score|English]]''' - '''[[${vote_wiki}${detail_wiki}_par_note|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text par note finale==

MEDIAWIKI_PAGE

$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
$mediawiki_page .= "! Eurodéputé(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
$mediawiki_page .= "|-\n";
foreach my $mep (sort sort_by_score keys %scores) {
  my @coloured_scores = get_coloured_score($mep);
  my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
  my $id_mep = $scores{$mep}->{DBXMLID} ? "id='$scores{$mep}->{DBXMLID}'" : "id='$mep'";
  $id_mep =~ s/ /_/go;
  $mediawiki_page .= "|-$id_mep style='display:none'\n";
  $mediawiki_page .= "| $wiki_mep || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
}
$mediawiki_page .= "|}\n";
if ($detail) {
  $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_par_note Voir les détails pour tous les eurodéputés]\n";
  $mediawiki_page .= "\n----\n\n$details\n";
  $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_note|Revenir aux résultats par note finale]]\n";
}
else {
  $mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
}
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by score
$mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_par_note.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build global mediawiki page by score
if ($detail) {
  $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}_by_score|English]]''' - '''[[${vote_wiki}_par_note|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par note finale==

* [[${vote_wiki}${detail_wiki}_par_note|Détail des notes]]

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Rang !! Eurodéputé(e) !! Pays !! Groupe !! Note finale !!\n";
  $mediawiki_page .= "|-\n";
  my $rank = 1;
  foreach my $mep (sort sort_by_score keys %scores) {
    my @coloured_scores = get_coloured_score($mep);
    my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
    my $show = $scores{$mep}->{DBXMLID} ? "?showmep=$scores{$mep}->{DBXMLID}" : "?showmep=$mep";
    $show =~ s/ /_/go;
    my $country = $scores{$mep}->{DBXMLID} ? "{{#icon:$scores{$mep}->{COUNTRY}->{CODE}.png|$scores{$mep}->{COUNTRY}->{NAME}||${vote_wiki}_$scores{$mep}->{COUNTRY}->{CODE}}} [[${vote_wiki}_$scores{$mep}->{COUNTRY}->{CODE}|$scores{$mep}->{COUNTRY}->{NAME}]]" : '?';
    my $grp_wiki = $scores{$mep}->{DBXMLID} ? $scores{$mep}->{GRP}->{CODE} : '?';
    $grp_wiki =~ s/[\/-].+$//o;
    my $grp = $scores{$mep}->{DBXMLID} ? "{{#icon:$grp_wiki.png|$scores{$mep}->{GRP}->{CODE}||${vote_wiki}_$grp_wiki}} [[${vote_wiki}_$grp_wiki|$scores{$mep}->{GRP}->{CODE}]]" : '?';
    $mediawiki_page .= "| $rank || $wiki_mep || $country || $grp || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_par_note$show Voir les details]\n";
    $mediawiki_page .= "|-\n";
    $rank++;
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
  $mediawiki_page .= "$categories{$vote_wiki}\n";

  # Output global mediawiki page by score
  $mediawiki_location = "$output_dir/${vote_wiki}_par_note.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

# Build mediawiki page by name
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}${detail_wiki}_by_name|English]]''' - '''[[${vote_wiki}${detail_wiki}_par_nom|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text par ordre alphabétique==

MEDIAWIKI_PAGE

$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
$mediawiki_page .= "! Eurodéputé(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
foreach my $mep (sort sort_by_name keys %scores) {
  my @coloured_scores = get_coloured_score($mep);
  my $id_mep = $scores{$mep}->{DBXMLID} ? "id='$scores{$mep}->{DBXMLID}'" : "id='$mep'";
  $id_mep =~ s/ /_/go;
  $mediawiki_page .= "|-$id_mep style='display:none'\n";
  my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
  $mediawiki_page .= "| $wiki_mep || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
}
$mediawiki_page .= "|}\n";
if ($detail) {
  $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_par_nom Show details for all MEPs]\n";
  $mediawiki_page .= "\n----\n\n$details\n";
  $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_nom|Revenir aux résultats par ordre alphabétique]]\n";
}
else {
  $mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
}
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by score
$mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_par_nom.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build global mediawiki page by name
if ($detail) {
  $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}_by_name|English]]''' - '''[[${vote_wiki}_par_nom|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par ordre alphabétique==

* [[${vote_wiki}${detail_wiki}_par_nom|Détail des notes]]

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Rang !! Eurodéputé(e) !! Pays !! Groupe !! Note finale !!\n";
  $mediawiki_page .= "|-\n";
  my $rank = 1;
  foreach my $mep (sort sort_by_name keys %scores) {
    my @coloured_scores = get_coloured_score($mep);
    my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
    my $show = $scores{$mep}->{DBXMLID} ? "?showmep=$scores{$mep}->{DBXMLID}" : "?showmep=$mep";
    $show =~ s/ /_/go;
    my $country = $scores{$mep}->{DBXMLID} ? "{{#icon:$scores{$mep}->{COUNTRY}->{CODE}.png|$scores{$mep}->{COUNTRY}->{NAME}||${vote_wiki}_$scores{$mep}->{COUNTRY}->{CODE}}} [[${vote_wiki}_$scores{$mep}->{COUNTRY}->{CODE}|$scores{$mep}->{COUNTRY}->{NAME}]]" : '?';
    my $grp_wiki = $scores{$mep}->{DBXMLID} ? $scores{$mep}->{GRP}->{CODE} : '?';
    $grp_wiki =~ s/[\/-].+$//o;
    my $grp = $scores{$mep}->{DBXMLID} ? "{{#icon:$grp_wiki.png|$scores{$mep}->{GRP}->{CODE}||${vote_wiki}_$grp_wiki}} [[${vote_wiki}_$grp_wiki|$scores{$mep}->{GRP}->{CODE}]]" : '?';
    $mediawiki_page .= "| $rank || $wiki_mep || $country || $grp || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_par_nom$show Voir les détails]\n";
    $mediawiki_page .= "|-\n";
    $rank++;
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
  $mediawiki_page .= "$categories{$vote_wiki}\n";

  # Output mediawiki page by score
  $mediawiki_location = "$output_dir/${vote_wiki}_par_nom.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

# Build mediawiki pages for each country
my %country_meps = map {$_->{COUNTRY}->{CODE} => $_->{COUNTRY}->{NAME}} grep {$_->{COUNTRY}} values %scores;
my %country_mean;
foreach my $code (sort keys %country_meps) {
  my $preposition = $country_meps{$code} =~ /^[AEIOUY]/o ? "l'" : $country_meps{$code} =~ /^(?:DK|GB|LU|PT)$/o ? "le " : $country_meps{$code} =~ /^(?:CY|MT)$/o ? '' : $country_meps{$code} eq 'NL' ? "les " : "la ";
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}${detail_wiki}_$code|English]]''' - '''[[${vote_wiki}${detail_wiki}_$code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text pour $preposition$country_meps{$code} {{#icon:$code.png|$country_meps{$code}}}==

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Eurodéputé(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
  my $country_nb = 0;
  my $country_total = 0;
  foreach my $mep (sort sort_by_score grep {$scores{$_}->{COUNTRY} and $scores{$_}->{COUNTRY}->{CODE} eq $code} keys %scores) {
    my @coloured_scores = get_coloured_score($mep);
    my $id_mep = $scores{$mep}->{DBXMLID} ? "id='$scores{$mep}->{DBXMLID}'" : "id='$mep'";
    $id_mep =~ s/ /_/go;
    $mediawiki_page .= "|-$id_mep style='display:none'\n";
    my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
    $mediawiki_page .= "| $wiki_mep || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
    $country_nb++;
    $country_total += $scores{$mep}->{FINAL_SCORE};
  }
  $mediawiki_page .= "|}\n";
  if ($detail) {
    $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$code Voir les détails pour tous les eurodéputés]\n";
    $mediawiki_page .= "\n----\n\n$details\n";
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_$code|Revenir aux résultats pour $preposition$country_meps{$code}]]\n";
  }
  else {
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_pays|Autres pays]]\n";
  }
  $mediawiki_page .= "$categories{$vote_wiki}\n";
  $country_mean{$code} = sprintf("%.1f", $country_total / $country_nb);

  # Output mediawiki page for this country
  my $mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_$code.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";

  # Build global mediawiki page for this country
  if ($detail) {
    my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}_$code|English]]''' - '''[[${vote_wiki}_$code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats pour $preposition$country_meps{$code} {{#icon:$code.png|$country_meps{$code}}}==

* [[${vote_wiki}${detail_wiki}_$code|Détail des notes]]

MEDIAWIKI_PAGE

    $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
    $mediawiki_page .= "! Rang !! Eurodéputé(e) !! Groupe !! Note finale !!\n";
    $mediawiki_page .= "|-\n";
    my $rank = 1;
    foreach my $mep (sort sort_by_score grep {$scores{$_}->{COUNTRY} and $scores{$_}->{COUNTRY}->{CODE} eq $code} keys %scores) {
      my @coloured_scores = get_coloured_score($mep);
      my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
      my $show = $scores{$mep}->{DBXMLID} ? "?showmep=$scores{$mep}->{DBXMLID}" : "?showmep=$mep";
      $show =~ s/ /_/go;
      my $grp_wiki = $scores{$mep}->{DBXMLID} ? $scores{$mep}->{GRP}->{CODE} : '?';
    $grp_wiki =~ s/[\/-].+$//o;
      my $grp = $scores{$mep}->{DBXMLID} ? "{{#icon:$grp_wiki.png|$scores{$mep}->{GRP}->{CODE}||${vote_wiki}_$grp_wiki}} [[${vote_wiki}_$grp_wiki|$scores{$mep}->{GRP}->{CODE}]]" : '?';
      $mediawiki_page .= "| $rank || $wiki_mep || $grp || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$code$show Voir les détails]\n";
      $mediawiki_page .= "|-\n";
      $rank++;
    }
    $mediawiki_page =~ s/\|-\n$/|}\n/so;
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_pays|Autres pays]]\n";
    $mediawiki_page .= "$categories{$vote_wiki}\n";

    # Output mediawiki page for this country
    my $mediawiki_location = "$output_dir/${vote_wiki}_$code.mediawiki";
    open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
    print MEDIAWIKI $mediawiki_page;
    close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
    warn "$mediawiki_location done\n";
  }
}

# Build mediawiki page by country
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}_by_country|English]]''' - '''[[${vote_wiki}_par_pays|Français]]'''</td></tr></table></div>

{|- style="background:none;"
==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par pays==

MEDIAWIKI_PAGE

my $nb_col = 1;
my $col = 0;
foreach my $country_code (sort  {$country_mean{$b} <=> $country_mean{$a}} keys %country_meps) {
  $mediawiki_page .= "| {{#icon:$country_code.png|$country_meps{$country_code}||${vote_wiki}_$country_code}} [[${vote_wiki}_$country_code|$country_meps{$country_code}]]: mean score: $country_mean{$country_code}/100\n";
  $col++;
  if ($col == $nb_col) {
    $mediawiki_page .= "|-\n";
    $col = 0;
  }
}
$mediawiki_page =~ s/|-\n$//o;
$mediawiki_page .= "|}\n";
$mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by country
$mediawiki_location = "$output_dir/${vote_wiki}_par_pays.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki pages for each group
my %group_meps = map {my $grp = $_->{GRP}->{CODE}; $grp =~ s/[\/-].+$//o; $grp => $_->{GRP}->{NAME}} grep {$_->{GRP}} values %scores;
my %group_mean;
foreach my $code (sort keys %group_meps) {
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}${detail_wiki}_$code|English]]''' - '''[[${vote_wiki}${detail_wiki}_$code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text pour le groupe $group_meps{$code} {{#icon:$code.png|$group_meps{$code}}}==

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Eurodéputé(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
  my $group_nb = 0;
  my $group_total = 0;
  foreach my $mep (sort sort_by_score grep {$scores{$_}->{GRP} and $scores{$_}->{GRP}->{CODE} =~ /^$code/} keys %scores) {
    my @coloured_scores = get_coloured_score($mep);
    my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
    my $id_mep = $scores{$mep}->{DBXMLID} ? "id='$scores{$mep}->{DBXMLID}'" : "id='$mep'";
    $id_mep =~ s/ /_/go;
    $mediawiki_page .= "|-$id_mep style='display:none'\n";
    $mediawiki_page .= "| $wiki_mep || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
    $group_nb++;
    $group_total += $scores{$mep}->{FINAL_SCORE};
  }
  $mediawiki_page .= "|}\n";
  if ($detail) {
    $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$code Voir les détails pour tous les eurodéputés]\n";
    $mediawiki_page .= "\n----\n\n$details\n";
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_$code|Revenir aux résultats pour le groupe $group_meps{$code}]]\n";
  }
  else {
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_groupe|Autres groupes politiques]]\n";
  }
  $mediawiki_page .= "$categories{$vote_wiki}\n";
  $group_mean{$code} = sprintf("%.1f", $group_total / $group_nb);

  # Output mediawiki page for this group
  my $mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_$code.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";

  # Build global mediawiki page for this group
  if ($detail) {
    my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}_$code|English]]''' - '''[[${vote_wiki}_$code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats pour le groupe $group_meps{$code} {{#icon:$code.png|$group_meps{$code}}}==

* [[${vote_wiki}${detail_wiki}_$code|Détail des notes]]

MEDIAWIKI_PAGE

    $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
    $mediawiki_page .= "! Rang !! Eurodéputé(e) !! Pays !! Note finale !!\n";
    $mediawiki_page .= "|-\n";
    my $rank = 1;
    foreach my $mep (sort sort_by_score grep {$scores{$_}->{GRP} and $scores{$_}->{GRP}->{CODE} =~ /^$code/} keys %scores) {
      my @coloured_scores = get_coloured_score($mep);
      my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
      my $show = $scores{$mep}->{DBXMLID} ? "?showmep=$scores{$mep}->{DBXMLID}" : "?showmep=$mep";
      $show =~ s/ /_/go;
      my $country = $scores{$mep}->{DBXMLID} ? "{{#icon:$scores{$mep}->{COUNTRY}->{CODE}.png|$scores{$mep}->{COUNTRY}->{NAME}||${vote_wiki}_$scores{$mep}->{COUNTRY}->{CODE}}} [[${vote_wiki}_$scores{$mep}->{COUNTRY}->{CODE}|$scores{$mep}->{COUNTRY}->{NAME}]]" : '?';
      $mediawiki_page .= "| $rank || $wiki_mep || $country || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$code$show Voir les détails]\n";
      $mediawiki_page .= "|-\n";
      $rank++;
    }
    $mediawiki_page =~ s/\|-\n$/|}\n/so;
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_groupe|Autres groupes politiques]]\n";
    $mediawiki_page .= "$categories{$vote_wiki}\n";

    # Output mediawiki page for this group
    my $mediawiki_location = "$output_dir/${vote_wiki}_$code.mediawiki";
    open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
    print MEDIAWIKI $mediawiki_page;
    close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
    warn "$mediawiki_location done\n";
  }
}

# Build mediawiki page by group
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}_by_group|English]]''' - '''[[${vote_wiki}_par_groupe|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par groupe politique==

{|- style="background:none;"
MEDIAWIKI_PAGE

$nb_col = 1;
$col = 0;
foreach my $group_code (sort { $group_mean{$b} <=> $group_mean{$a} } keys %group_meps) {
  (my $group_wiki = $group_code) =~ s/[\/-].+$//o;
  $mediawiki_page .= "| {{#icon:$group_wiki.png|$group_meps{$group_code}||${vote_wiki}_$group_code}}\n";
  $mediawiki_page .= "| [[${vote_wiki}_$group_code|$group_meps{$group_code}]]: mean score: $group_mean{$group_code}/100\n";
  $col++;
  if ($col == $nb_col) {
    $mediawiki_page .= "|-\n";
    $col = 0;
  }
}
$mediawiki_page =~ s/|-\n$//o;
$mediawiki_page .= "|}\n";
$mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by group
$mediawiki_location = "$output_dir/${vote_wiki}_par_groupe.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki pages for each committee
my %com_mean;
my %com_meps = map {%{$_->{COMS}}} grep {exists $_->{COMS}} values %scores;
foreach my $code (sort keys %com_meps) {
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}${detail_wiki}_$code|English]]''' - '''[[${vote_wiki}${detail_wiki}_$code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text pour la $com_meps{$code}==

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Eurodéputé(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
  my $com_nb = 0;
  my $com_total = 0;
  foreach my $mep (sort sort_by_score grep {$scores{$_}->{COMS} and exists $scores{$_}->{COMS}->{$code}} keys %scores) {
    my @coloured_scores = get_coloured_score($mep);
    my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
    my $id_mep = $scores{$mep}->{DBXMLID} ? "id='$scores{$mep}->{DBXMLID}'" : "id='$mep'";
    $id_mep =~ s/ /_/go;
    $mediawiki_page .= "|-$id_mep style='display:none'\n";
    $mediawiki_page .= "| $wiki_mep || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
    $com_nb++;
    $com_total += $scores{$mep}->{FINAL_SCORE};
  }
  $mediawiki_page .= "|}\n";
  if ($detail) {
    $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$code Voir les détails pour tous les eurodéputés]\n";
    $mediawiki_page .= "\n----\n\n$details\n";
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_$code|Revenir aux résultats pour la $com_meps{$code}]]\n";
  }
  else {
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_commission|Autres commissions parlementaires]]\n";
  }
  $mediawiki_page .= "$categories{$vote_wiki}\n";
  $com_mean{$code} = sprintf("%.1f", $com_total / $com_nb);

  # Output mediawiki page for this committee
  my $mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_$code.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";

  # Build global mediawiki page for this committee
  if ($detail) {
    my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}_$code|English]]''' - '''[[${vote_wiki}_$code|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats pour la $com_meps{$code}==

* [[${vote_wiki}${detail_wiki}_$code|Détail des notes]]

MEDIAWIKI_PAGE

    $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
    $mediawiki_page .= "! Rang !! Eurodéputé(e) !! Pays !! Groupe !! Note finale !!\n";
    $mediawiki_page .= "|-\n";
    my $rank = 1;
    foreach my $mep (sort sort_by_score grep {$scores{$_}->{COMS} and exists $scores{$_}->{COMS}->{$code}} keys %scores) {
      my @coloured_scores = get_coloured_score($mep);
      my $wiki_mep = $scores{$mep}->{DBXMLID} ? "[[$scores{$mep}->{DBXMLID}/fr|$mep]]" : $mep;
      my $show = $scores{$mep}->{DBXMLID} ? "?showmep=$scores{$mep}->{DBXMLID}" : "?showmep=$mep";
      $show =~ s/ /_/go;
      my $country = $scores{$mep}->{DBXMLID} ? "{{#icon:$scores{$mep}->{COUNTRY}->{CODE}.png|$scores{$mep}->{COUNTRY}->{NAME}||${vote_wiki}_$scores{$mep}->{COUNTRY}->{CODE}}} [[${vote_wiki}_$scores{$mep}->{COUNTRY}->{CODE}|$scores{$mep}->{COUNTRY}->{NAME}]]" : '?';
      my $grp_wiki = $scores{$mep}->{DBXMLID} ? $scores{$mep}->{GRP}->{CODE} : '?';
      $grp_wiki =~ s/[\/-].+$//o;
      my $grp = $scores{$mep}->{DBXMLID} ? "{{#icon:$grp_wiki.png|$scores{$mep}->{GRP}->{CODE}||${vote_wiki}_$grp_wiki}} [[${vote_wiki}_$grp_wiki|$scores{$mep}->{GRP}->{CODE}]]" : '?';
      $mediawiki_page .= "| $rank || $wiki_mep || $country || $grp || style=\"background:".$colours[sprintf("%.0f", $scores{$mep}->{FINAL_SCORE}/5)]."\" | $scores{$mep}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$code$show Voir les détails]\n";
      $mediawiki_page .= "|-\n";
      $rank++;
    }
    $mediawiki_page =~ s/\|-\n$/|}\n/so;
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_commission|Autres commissions parlementaires]]\n";
    $mediawiki_page .= "$categories{$vote_wiki}\n";

    # Output mediawiki page for this committee
    my $mediawiki_location = "$output_dir/${vote_wiki}_$code.mediawiki";
    open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
    print MEDIAWIKI $mediawiki_page;
    close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
    warn "$mediawiki_location done\n";
  }
}

# Build mediawiki page by committee
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$translations{$vote_wiki}_by_committee|English]]''' - '''[[${vote_wiki}_par_commission|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par commission parlementaire==

MEDIAWIKI_PAGE

$mediawiki_page .= join "\n", map {"* [[${vote_wiki}_$_|$com_meps{$_}]] : note moyenne : $com_mean{$_}/100"} sort {$com_mean{$b} <=> $com_mean{$a}} keys %com_meps;
$mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by committee
$mediawiki_location = "$output_dir/${vote_wiki}_par_commission.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

sub get_coloured_score {
  my $mep = shift;
  my @coloured_scores;

  foreach my $part (@parts) {
    my $colour = '#ffffff';
    if ($votes{VOTES}->{$part}->{REC}) {
      if ($scores{$mep}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT} == 2) {
        $colour = '#00ff00'
      }
      elsif ($scores{$mep}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT} == 1) {
        $colour = '#008800'
      }
      elsif ($scores{$mep}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT} == -1) {
        $colour = '#dc143c'
      }
      elsif ($scores{$mep}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT} == -2) {
        $colour = '#ff0000'
      }
    }

    my $choice = '-';
    if ($scores{$mep}->{VOTES}->{$part}->{VOTE}) {
      if ($scores{$mep}->{VOTES}->{$part}->{VOTE} eq 'for') {
        $choice = 'pour';
      }
      elsif ($scores{$mep}->{VOTES}->{$part}->{VOTE} eq 'against') {
        $choice = 'contre';
      }
      elsif ($scores{$mep}->{VOTES}->{$part}->{VOTE} eq 'abstention') {
        $choice = 'abst.';
      }
    }

    push @coloured_scores, "style=\"background:$colour\" | $choice";
  }
  return @coloured_scores;
}

sub sort_by_score {
  $scores{$b}->{FINAL_SCORE} <=> $scores{$a}->{FINAL_SCORE} ||
  (
    $scores{$a}->{NAME} ?
      (
        $scores{$b}->{NAME} ?
          (
            $scores{$a}->{NAME}->{LAST} cmp $scores{$b}->{NAME}->{LAST} ||
            $scores{$a}->{NAME}->{FIRST} cmp $scores{$b}->{NAME}->{FIRST}
          ) :
        $scores{$a}->{NAME}->{LAST} cmp $b
      ) :
      (
        $scores{$b}->{NAME} ?
          $a cmp $scores{$b}->{NAME}->{LAST} :
          $a cmp $b
      )
  )
}

sub sort_by_name {
  $scores{$a}->{NAME} ?
    (
      $scores{$b}->{NAME} ?
        (
          $scores{$a}->{NAME}->{LAST} cmp $scores{$b}->{NAME}->{LAST} ||
          $scores{$a}->{NAME}->{FIRST} cmp $scores{$b}->{NAME}->{FIRST}
        ) :
      $scores{$a}->{NAME}->{LAST} cmp $b
    ) :
    (
      $scores{$b}->{NAME} ?
        $a cmp $scores{$b}->{NAME}->{LAST} :
        $a cmp $b
    )
}

sub contains {
  my $array = shift;
  my $elt = shift;

  foreach my $this_elt (@$array) {
    return 1 if $this_elt eq $elt;
  }
  return 0;
}
